import re

import CoreFoundation
from PyObjCTools.TestSupport import TestCase


class TestCFUUIDAPI(TestCase):
    def testTypes(self):
        self.assertIsCFType(CoreFoundation.CFUUIDRef)

    def testTypeID(self):
        v = CoreFoundation.CFUUIDGetTypeID()
        self.assertIsInstance(v, int)

    def testCreate(self):
        self.assertResultIsCFRetained(CoreFoundation.CFUUIDCreate)
        uuid = CoreFoundation.CFUUIDCreate(None)
        self.assertIsNot(uuid, None)
        self.assertIsInstance(uuid, CoreFoundation.CFUUIDRef)
        text = CoreFoundation.CFUUIDCreateString(None, uuid)
        self.assertIsInstance(text, str)
        m = re.match("^[0-9A-Z]{8}(-[0-9A-Z]{4}){3}-[0-9A-Z]{12}$", text)
        self.assertIsNot(m, None)

    def testCreateWithBytes(self):
        self.assertResultIsCFRetained(CoreFoundation.CFUUIDCreateWithBytes)
        uuid = CoreFoundation.CFUUIDCreateWithBytes(
            None, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16
        )
        self.assertIsNot(uuid, None)
        self.assertIsInstance(uuid, CoreFoundation.CFUUIDRef)
        self.assertResultIsCFRetained(CoreFoundation.CFUUIDCreateString)
        text = CoreFoundation.CFUUIDCreateString(None, uuid)
        self.assertEqual(text, "01020304-0506-0708-090A-0B0C0D0E0F10")
        self.assertRaises(
            ValueError,
            CoreFoundation.CFUUIDCreateWithBytes,
            None,
            1,
            2,
            3,
            4,
            5,
            6,
            7,
            8,
            9,
            10,
            11,
            12,
            13,
            14,
            15,
            300,
        )
        self.assertRaises(
            ValueError,
            CoreFoundation.CFUUIDCreateWithBytes,
            None,
            1,
            2,
            3,
            4,
            5,
            6,
            7,
            8,
            9,
            10,
            11,
            12,
            13,
            14,
            300,
            16,
        )

    def testCreateFromString(self):
        self.assertResultIsCFRetained(CoreFoundation.CFUUIDCreateFromString)
        uuid1 = CoreFoundation.CFUUIDCreateFromString(
            None, "01020304-0506-0708-090A-0B0C0D0E0F10"
        )
        self.assertIsNot(uuid1, None)
        self.assertIsInstance(uuid1, CoreFoundation.CFUUIDRef)
        text = CoreFoundation.CFUUIDCreateString(None, uuid1)
        self.assertEqual(text, "01020304-0506-0708-090A-0B0C0D0E0F10")
        uuid2 = CoreFoundation.CFUUIDCreateFromString(
            None, "01020304-0506-0708-090A-0B0C0D0E0F10"
        )
        text = CoreFoundation.CFUUIDCreateString(None, uuid2)
        self.assertEqual(text, "01020304-0506-0708-090A-0B0C0D0E0F10")
        # CoreFoundation.CFUUID interns values
        self.assertIs(uuid1, uuid2)

    def testGetBytes(self):
        uuid = CoreFoundation.CFUUIDCreateWithBytes(
            None, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16
        )
        self.assertIsNot(uuid, None)
        self.assertIsInstance(uuid, CoreFoundation.CFUUIDRef)
        bytes_value = CoreFoundation.CFUUIDGetUUIDBytes(uuid)
        self.assertIsInstance(bytes_value, CoreFoundation.CFUUIDBytes)
        self.assertEqual(bytes_value.byte0, 1)
        self.assertEqual(bytes_value.byte1, 2)
        self.assertEqual(bytes_value.byte2, 3)
        self.assertEqual(bytes_value.byte3, 4)
        self.assertEqual(bytes_value.byte4, 5)
        self.assertEqual(bytes_value.byte5, 6)
        self.assertEqual(bytes_value.byte6, 7)
        self.assertEqual(bytes_value.byte7, 8)
        self.assertEqual(bytes_value.byte8, 9)
        self.assertEqual(bytes_value.byte9, 10)
        self.assertEqual(bytes_value.byte10, 11)
        self.assertEqual(bytes_value.byte11, 12)
        self.assertEqual(bytes_value.byte12, 13)
        self.assertEqual(bytes_value.byte13, 14)
        self.assertEqual(bytes_value.byte14, 15)
        self.assertEqual(bytes_value.byte15, 16)

    def testConstant(self):
        # This is an interesting one, the result of
        # CoreFoundation.CFUUIDGetConstantUUIDWithBytes should not be released.

        uuid = CoreFoundation.CFUUIDGetConstantUUIDWithBytes(None, *range(16))
        CoreFoundation.CFRetain(
            CoreFoundation.CFUUIDGetConstantUUIDWithBytes
        )  # Ensure the value won't be released.
        self.assertIsNot(uuid, None)
        self.assertIsInstance(uuid, CoreFoundation.CFUUIDRef)
        s = CoreFoundation.CFUUIDCreateString(None, uuid)

        uuid = None
        del uuid

        uuid = CoreFoundation.CFUUIDGetConstantUUIDWithBytes(None, *range(16))
        self.assertIsNot(uuid, None)
        self.assertIsInstance(uuid, CoreFoundation.CFUUIDRef)
        t = CoreFoundation.CFUUIDCreateString(None, uuid)

        self.assertEqual(s, t)

    def testCreateFromUUIDBytes(self):
        bytes_value = CoreFoundation.CFUUIDBytes(*range(16, 32))
        uuid = CoreFoundation.CFUUIDCreateFromUUIDBytes(None, bytes_value)

        self.assertIsNot(uuid, None)
        self.assertIsInstance(uuid, CoreFoundation.CFUUIDRef)
        text = CoreFoundation.CFUUIDCreateString(None, uuid)
        self.assertEqual(text, "10111213-1415-1617-1819-1A1B1C1D1E1F")

    def testStructs(self):
        o = CoreFoundation.CFUUIDBytes()
        self.assertHasAttr(o, "byte0")
        self.assertHasAttr(o, "byte1")
        self.assertHasAttr(o, "byte2")
        self.assertHasAttr(o, "byte3")
        self.assertHasAttr(o, "byte4")
        self.assertHasAttr(o, "byte5")
        self.assertHasAttr(o, "byte6")
        self.assertHasAttr(o, "byte7")
        self.assertHasAttr(o, "byte8")
        self.assertHasAttr(o, "byte9")
        self.assertHasAttr(o, "byte10")
        self.assertHasAttr(o, "byte11")
        self.assertHasAttr(o, "byte12")
        self.assertHasAttr(o, "byte13")
        self.assertHasAttr(o, "byte14")
        self.assertHasAttr(o, "byte15")



#
# Demonstrates that the super-class implementation of an overridden method
# can be called in the same way as with normal objects.
#
from Foundation import NSObject
from objc import super

N = 1


class MyObject(NSObject):
    def init(self):
        global N
        if N == 1:
            print("Calling super.init")
            N = 0

            # Call super-class implementation.
            super().init()

        else:
            print("Cyclic call detected")


x = MyObject.alloc().init()



import AppKit
from PyObjCTools.TestSupport import TestCase


class TestNSWindowController(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSWindowController.shouldCascadeWindows)
        self.assertArgIsBOOL(AppKit.NSWindowController.setShouldCascadeWindows_, 0)
        self.assertArgIsBOOL(AppKit.NSWindowController.setDocumentEdited_, 0)
        self.assertResultIsBOOL(AppKit.NSWindowController.shouldCloseDocument)
        self.assertArgIsBOOL(AppKit.NSWindowController.setShouldCloseDocument_, 0)
        self.assertResultIsBOOL(AppKit.NSWindowController.isWindowLoaded)



"""
Script for building the example.

Usage:
    python3 setup.py py2app
"""
from setuptools import setup

plist = {"NSMainNibFile": "OpenGLDemo"}
setup(
    name="OpenGLDemo",
    app=["OpenGLDemo.py"],
    data_files=["OpenGLDemo.nib"],
    options={"py2app": {"plist": plist}},
    setup_requires=["py2app", "PyOpenGL", "pyobjc-framework-Cocoa"],
)



import Cocoa
import objc
from objc import super

# enum ToDoItemStatus
INCOMPLETE = 0
COMPLETE = 1
DEFER_TO_NEXT_DAY = 2

SECS_IN_MINUTE = 60
SECS_IN_HOUR = SECS_IN_MINUTE * 60
SECS_IN_DAY = SECS_IN_HOUR * 24
SECS_IN_WEEK = SECS_IN_DAY * 7


class ToDoItem(Cocoa.NSObject):
    __slots__ = (
        "_day",
        "_itemName",
        "_notes",
        "_timer",
        "_secsUntilDue",
        "_secsUntilNotify",
        "_status",
    )

    def init(self):
        self = super().init()
        if self is None:
            return None

        self._day = None
        self._itemName = None
        self._notes = None
        self._secsUntilDue = 0
        self._secsUntilNotify = 0
        self._status = None
        self._timer = None

    def description(self):
        descr = """%s
\tName: %s
\tDay: %s
\tNotes: %s
\tCompleted: %s
\tSecs Until Due: %d
\tSecs Until Notify: %d
""" % (
            super.description(),
            self.itemName(),
            self._day,
            self._notes,
            ["No", "YES"][self.status() == COMPLETE],
            self._secsUntilDue,
            self._secsUntilNotify,
        )
        return descr

    def initWithName_andDate_(self, aName, aDate):
        self = super().init()
        if self is None:
            return None

        self._day = None
        self._itemName = None
        self._notes = None
        self._secsUntilDue = 0
        self._secsUntilNotify = 0
        self._status = None
        self._timer = None

        if not aName:
            return None

        self.setItemName_(aName)

        if aDate:
            self.setDay_(aDate)
        else:
            now = Cocoa.NSCalendarDate.date()

            self.setDay_(
                Cocoa.NSCalendarDate.dateWithYear_month_day_hour_minute_second_timeZone_(
                    now.yearOfCommonEra(),
                    now.monthOfYear(),
                    now.dayOfMonth(),
                    0,
                    0,
                    0,
                    Cocoa.NSTimeZone.localTimeZone(),
                )
            )
        self.setStatus_(INCOMPLETE)
        self.setNotes_("")
        return self

    def encodeWithCoder_(self, coder):

        coder.encodeObject_(self._day)
        coder.encodeObject_(self._itemName)
        coder.encodeObject_(self._notes)

        tempTime = self._secsUntilDue
        coder.encodeValueOfObjCType_at_(objc._C_LNG, tempTime)

        tempTime = self._secsUntilNotify
        coder.encodeValueOfObjCType_at_(objc._C_LNG, tempTime)

        tempStatus = self._status
        coder.encodeValueOfObjCType_at_(objc._C_INT, tempStatus)

    def initWithCoder_(self, coder):

        self.setDay_(coder.decodeObject())
        self.setItemName_(coder.decodeObject())
        self.setNotes_(coder.decodeObject())

        tempTime = coder.decodeObjectOfObjCType_at_(objc._C_LNG)
        self.setSecsUntilDue_(tempTime)

        tempTime = coder.decodeObjectOfObjCType_at_(objc._C_LNG)
        self.setSecsUntilNotify_(tempTime)

        tempStatus = coder.decodeObjectOfObjCType_at_(objc._C_INT)
        self.setSecsUntilNotify_(tempStatus)

        return self

    def __del__(self):  # dealloc
        if self._notes:
            self._timer.invalidate()

    def setDay_(self, newDay):
        self._day = newDay

    def day(self):
        return self._day

    def setItemName_(self, newName):
        self._itemName = newName

    def itemName(self):
        return self._itemName

    def setNotes_(self, newNotes):
        self._notes = newNotes

    def notes(self):
        return self._notes

    def setTimer_(self, newTimer):
        if self._timer:
            self._timer.invalidate()

        if newTimer:
            self._timer = newTimer
        else:
            self._timer = None

    def timer(self):
        return self._timer

    def setStatus_(self, newStatus):
        self._status = newStatus

    def status(self):
        return self._status

    def setSecsUntilDue_(self, secs):
        self._secsUntilDue = secs

    def secsUntilDue(self):
        return self._secsUntilDue

    def setSecsUntilNotify_(self, secs):
        self._secsUntilNotify = secs

    def secsUntilNotify(self):
        return self._secsUntilNotify


def ConvertTimeToSeconds(hour, minute, pm):
    if hour == 12:
        hour = 0

    if pm:
        hour += 12

    return (hour * SECS_IN_HOUR) + (minute * SECS_IN_MINUTE)


def ConvertSecondsToTime(secs):
    pm = 0

    hour = secs / SECS_IN_HOUR
    if hour > 11:
        hour -= 12
        pm = 1

    if hour == 0:
        hour = 12

    minute = (secs % SECS_IN_HOUR) / SECS_IN_MINUTE

    return (hour, minute, pm)



import Cocoa
import objc

searchIndex = 0


class MyWindowController(Cocoa.NSWindowController):
    query = objc.ivar()
    previousRowCount = objc.ivar(type=objc._C_INT)

    myTableView = objc.IBOutlet()
    mySearchResults = objc.IBOutlet()
    predicateEditor = objc.IBOutlet()
    progressView = objc.IBOutlet()  # the progress search view
    progressSearch = objc.IBOutlet()  # spinning gear
    progressSearchLabel = objc.IBOutlet()  # search result #

    def dealloc(self):
        Cocoa.NSNotificationCenter.defaultCenter().removeObserver_(self)

    def awakeFromNib(self):
        # no vertical scrolling, we always want to show all rows
        self.predicateEditor.enclosingScrollView().setHasVerticalScroller_(False)

        self.previousRowCount = 3
        self.predicateEditor.addRow_(self)

        # put the focus in the first text field
        displayValue = self.predicateEditor.displayValuesForRow_(1).lastObject()
        if isinstance(displayValue, Cocoa.NSControl):
            self.window().makeFirstResponder_(displayValue)

        # create and initialize our query
        self.query = Cocoa.NSMetadataQuery.alloc().init()

        # setup our Spotlight notifications
        nf = Cocoa.NSNotificationCenter.defaultCenter()
        nf.addObserver_selector_name_object_(
            self, "queryNotification:", None, self.query
        )

        # initialize our Spotlight query, sort by contact name

        self.query.setSortDescriptors_(
            [
                Cocoa.NSSortDescriptor.alloc().initWithKey_ascending_(
                    "kMDItemContactKeywords", True
                )
            ]
        )
        self.query.setDelegate_(self)

        # start with our progress search label empty
        self.progressSearchLabel.setStringValue_("")

        return

    def applicationShouldTerminateAfterLastWindowClosed_(self, sender):
        return True

    def loadResultsFromQuery_(self, notif):
        results = notif.object().results()

        Cocoa.NSLog("search count = %d", len(results))
        foundResultsStr = "Results found: %d" % (len(results),)
        self.progressSearchLabel.setStringValue_(foundResultsStr)

        if len(results) == 0:
            return

        # iterate through the array of results, and match to the existing stores
        for item in results:
            cityStr = item.valueForAttribute_("kMDItemCity")
            nameStr = item.valueForAttribute_("kMDItemDisplayName")
            stateStr = item.valueForAttribute_("kMDItemStateOrProvince")
            phoneNumbers = item.valueForAttribute_("kMDItemPhoneNumbers")
            phoneStr = None
            if phoneNumbers:
                phoneStr = phoneNumbers[0]

            storePath = item.valueForAttribute_(
                "kMDItemPath"
            ).stringByResolvingSymlinksInPath()

            # create a dictionary entry to be added to our search results array
            aDict = {
                "name": nameStr or "",
                "phone": phoneStr or "",
                "city": cityStr or "",
                "state": stateStr or "",
                "url": Cocoa.NSURL.fileURLWithPath_(storePath),
            }
            self.mySearchResults.append(aDict)

    def queryNotification_(self, note):
        # the NSMetadataQuery will send back a note when updates are happening.
        # By looking at the [note name], we can tell what is happening
        if note.name() == Cocoa.NSMetadataQueryDidStartGatheringNotification:
            # the query has just started
            Cocoa.NSLog("search: started gathering")

            self.progressSearch.setHidden_(False)
            self.progressSearch.startAnimation_(self)
            self.progressSearch.animate_(self)
            self.progressSearchLabel.setStringValue_("Searching...")

        elif note.name() == Cocoa.NSMetadataQueryDidFinishGatheringNotification:
            # at this point, the query will be done. You may receive an update
            # later on.
            Cocoa.NSLog("search: finished gathering")

            self.progressSearch.setHidden_(True)
            self.progressSearch.stopAnimation_(self)

            self.loadResultsFromQuery_(note)

        elif note.name() == Cocoa.NSMetadataQueryGatheringProgressNotification:
            # the query is still gathering results...
            Cocoa.NSLog("search: progressing...")

            self.progressSearch.animate_(self)

        elif note.name() == Cocoa.NSMetadataQueryDidUpdateNotification:
            # an update will happen when Spotlight notices that a file as
            # added, removed, or modified that affected the search results.
            Cocoa.NSLog("search: an update happened.")

    # -------------------------------------------------------------------------
    #   inspect:selectedObjects
    #
    #   This method obtains the selected object (in our case for single selection,
    #   it's the first item), and opens its URL.
    # -------------------------------------------------------------------------
    def inspect_(self, selectedObjects):
        objectDict = selectedObjects[0]
        if objectDict is not None:
            url = objectDict["url"]
            Cocoa.NSWorkspace.sharedWorkspace().openURL_(url)

    # ------------------------------------------------------------------------
    #   spotlightFriendlyPredicate:predicate
    #
    #   This method will "clean up" an NSPredicate to make it ready for Spotlight, or return nil
    #   if the predicate can't be cleaned.
    #
    #   Foundation's Spotlight support in NSMetdataQuery places the following requirements on
    #   an NSPredicate:
    #           - Value-type (always YES or NO) predicates are not allowed
    #           - Any compound predicate (other than NOT) must have at least two subpredicates
    # -------------------------------------------------------------------------
    def spotlightFriendlyPredicate_(self, predicate):
        if predicate == Cocoa.NSPredicate.predicateWithValue_(
            True
        ) or predicate == Cocoa.NSPredicate.predicateWithValue_(False):
            return False

        elif isinstance(predicate, Cocoa.NSCompoundPredicate):

            predicate_type = predicate.compoundPredicateType()
            cleanSubpredicates = []
            for dirtySubpredicate in predicate.subpredicates():
                cleanSubpredicate = self.spotlightFriendlyPredicate_(dirtySubpredicate)
                if cleanSubpredicate:
                    cleanSubpredicates.append(cleanSubpredicate)

            if len(cleanSubpredicates) == 0:
                return None

            else:
                if (
                    len(cleanSubpredicates) == 1
                    and predicate_type != Cocoa.NSNotPredicateType
                ):
                    return cleanSubpredicates[0]

                else:
                    return (
                        Cocoa.NSCompoundPredicate.alloc().initWithType_subpredicates_(
                            predicate_type, cleanSubpredicates
                        )
                    )

        else:
            return predicate

    # -------------------------------------------------------------------------
    #   createNewSearchForPredicate:predicate:withTitle
    #
    # -------------------------------------------------------------------------
    def createNewSearchForPredicate_withTitle_(self, predicate, title):
        if predicate is not None:
            self.mySearchResults.removeObjects_(self.mySearchResults.arrangedObjects())
            # remove the old search results

            # always search for items in the Address Book
            addrBookPredicate = Cocoa.NSPredicate.predicateWithFormat_(
                "(kMDItemKind = 'Address Book Person Data')"
            )
            predicate = Cocoa.NSCompoundPredicate.andPredicateWithSubpredicates_(
                [addrBookPredicate, predicate]
            )

            self.query.setPredicate_(predicate)
            self.query.startQuery()

    # --------------------------------------------------------------------------
    #   predicateEditorChanged:sender
    #
    #  This method gets called whenever the predicate editor changes.
    #   It is the action of our predicate editor and the single plate for all our updates.
    #
    #   We need to do potentially three things:
    #           1) Fire off a search if the user hits enter.
    #           2) Add some rows if the user deleted all of them, so the user isn't left
    #              without any rows.
    #           3) Resize the window if the number of rows changed (the user hit + or -).
    # --------------------------------------------------------------------------
    @objc.IBAction
    def predicateEditorChanged_(self, sender):
        # check NSApp currentEvent for the return key
        event = Cocoa.NSApp.currentEvent()
        if event is None:
            return

        if event.type() == Cocoa.NSKeyDown:
            characters = event.characters()
            if len(characters) > 0 and characters[0] == "\r":
                # get the predicat, which is the object value of our view
                predicate = self.predicateEditor.objectValue()

                # make it Spotlight friendly
                predicate = self.spotlightFriendlyPredicate_(predicate)
                if predicate is not None:
                    global searchIndex
                    title = Cocoa.NSLocalizedString("Search #%ld", "Search title")
                    self.createNewSearchForPredicate_withTitle_(
                        predicate, title % searchIndex
                    )
                    searchIndex += 1

        # if the user deleted the first row, then add it again - no sense
        # leaving the user with no rows
        if self.predicateEditor.numberOfRows() == 0:
            self.predicateEditor.addRow_(self)

        # resize the window vertically to accommodate our views:

        # get the new number of rows, which tells us the needed change in height,
        # note that we can't just get the view frame, because it's currently
        # animating - this method is called before the animation is finished.
        newRowCount = self.predicateEditor.numberOfRows()

        # if there's no change in row count, there's no need to resize anything
        if newRowCount == self.previousRowCount:
            return

        # The autoresizing masks, by default, allows the NSTableView to grow
        # and keeps the predicate editor fixed. We need to temporarily grow the
        # predicate editor, and keep the NSTableView fixed, so we have to change
        # the autoresizing masks.
        # Save off the old ones; we'll restore them after changing the window frame.
        tableScrollView = self.myTableView.enclosingScrollView()
        oldOutlineViewMask = tableScrollView.autoresizingMask()

        predicateEditorScrollView = self.predicateEditor.enclosingScrollView()
        oldPredicateEditorViewMask = predicateEditorScrollView.autoresizingMask()

        tableScrollView.setAutoresizingMask_(
            Cocoa.NSViewWidthSizable | Cocoa.NSViewMaxYMargin
        )
        predicateEditorScrollView.setAutoresizingMask_(
            Cocoa.NSViewWidthSizable | Cocoa.NSViewHeightSizable
        )

        # determine if we need to grow or shrink the window
        growing = newRowCount > self.previousRowCount

        # if growing, figure out by how much.  Sizes must contain nonnegative
        # values, which is why we avoid negative floats here.
        heightDifference = abs(
            self.predicateEditor.rowHeight() * (newRowCount - self.previousRowCount)
        )

        # convert the size to window coordinates -
        # if we didn't do this, we would break under scale factors other than 1.
        # We don't care about the horizontal dimension, so leave that as 0.
        #
        sizeChange = self.predicateEditor.convertSize_toView_(
            Cocoa.NSMakeSize(0, heightDifference), None
        )

        # offset our status view
        frame = self.progressView.frame()
        self.progressView.setFrameOrigin_(
            Cocoa.NSMakePoint(
                frame.origin.x,
                frame.origin.y
                - self.predicateEditor.rowHeight()
                * (newRowCount - self.previousRowCount),
            )
        )

        # change the window frame size:
        # - if we're growing, the height goes up and the origin goes down
        #    (corresponding to growing down).
        # - if we're shrinking, the height goes down and the origin goes up.
        windowFrame = self.window().frame()
        if growing:
            windowFrame.size.height += sizeChange.height
            windowFrame.origin.y -= sizeChange.height
        else:
            windowFrame.size.height -= sizeChange.height
            windowFrame.origin.y += sizeChange.height

        self.window().setFrame_display_animate_(windowFrame, True, True)

        # restore the autoresizing mask
        tableScrollView.setAutoresizingMask_(oldOutlineViewMask)
        predicateEditorScrollView.setAutoresizingMask_(oldPredicateEditorViewMask)

        self.previousRowCount = newRowCount  # save our new row count



import AppKit
from PyObjCTools.TestSupport import TestCase


class TestNSTabViewItem(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSSelectedTab, 0)
        self.assertEqual(AppKit.NSBackgroundTab, 1)
        self.assertEqual(AppKit.NSPressedTab, 2)

    def testMethods(self):
        self.assertArgIsBOOL(AppKit.NSTabViewItem.drawLabel_inRect_, 0)
        self.assertArgIsBOOL(AppKit.NSTabViewItem.sizeOfLabel_, 0)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level
import objc


class TestNSDraggingSession(TestCase):
    @min_os_level("10.7")
    def testMethods10_7(self):
        self.assertResultIsBOOL(
            AppKit.NSDraggingSession.animatesToStartingPositionsOnCancelOrFail
        )
        self.assertArgIsBOOL(
            AppKit.NSDraggingSession.setAnimatesToStartingPositionsOnCancelOrFail_, 0
        )

        self.assertArgIsBlock(
            AppKit.NSDraggingSession.enumerateDraggingItemsWithOptions_forView_classes_searchOptions_usingBlock_,  # noqa: B950
            4,
            b"v@" + objc._C_NSInteger + b"o^" + objc._C_NSBOOL,
        )



import AppKit
from PyObjCTools.TestSupport import TestCase


class ContantTest(TestCase):
    def testNSFloatingWindowLevel(self):
        # NSFloatingWindowLevel is a define in Objective-C, up-to 1.0rc1
        # we didn't correctly pick up this define due to a bug in the code.
        self.assertHasAttr(AppKit, "NSFloatingWindowLevel")
        self.assertIsInstance(AppKit.NSFloatingWindowLevel, int)

    def testNSAnyEventMask(self):
        self.assertEqual(AppKit.NSAnyEventMask, AppKit.NSUIntegerMax)

    def testNSViewFrameDidChangeNotification(self):
        self.assertHasAttr(AppKit, "NSViewFrameDidChangeNotification")
        self.assertIsInstance(AppKit.NSViewFrameDidChangeNotification, str)

    def testNSUpArrowFunctionKey(self):
        self.assertHasAttr(AppKit, "NSUpArrowFunctionKey")
        self.assertIsInstance(AppKit.NSUpArrowFunctionKey, str)



import Foundation
from PyObjCTools.TestSupport import TestCase


class TestXMLElement(TestCase):
    def testOutputArgs(self):
        self.assertArgIsOut(Foundation.NSXMLElement.initWithXMLString_error_, 1)
        self.assertArgIsBOOL(
            Foundation.NSXMLElement.normalizeAdjacentTextNodesPreservingCDATA_, 0
        )



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level
import objc


class TestNSTextRangeHelper(AppKit.NSObject):
    def compare_(self, a):
        pass


class TestNSTextRange(TestCase):
    def test_methods(self):
        self.assertResultHasType(TestNSTextRangeHelper.compare_, objc._C_NSInteger)

    @min_os_level("12.0")
    def test_methods12_0(self):
        self.assertResultIsBOOL(AppKit.NSTextRange.isEmpty)
        self.assertResultIsBOOL(AppKit.NSTextRange.isEqualToTextRange_)
        self.assertResultIsBOOL(AppKit.NSTextRange.containsLocation_)
        self.assertResultIsBOOL(AppKit.NSTextRange.containsRange_)
        self.assertResultIsBOOL(AppKit.NSTextRange.intersectsWithTextRange_)



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSURLError(TestCase):
    def testConstants(self):
        self.assertIsInstance(Foundation.NSURLErrorDomain, str)
        self.assertIsInstance(Foundation.NSErrorFailingURLStringKey, str)

        self.assertEqual(Foundation.NSURLErrorUnknown, -1)
        self.assertEqual(Foundation.NSURLErrorCancelled, -999)
        self.assertEqual(Foundation.NSURLErrorBadURL, -1000)
        self.assertEqual(Foundation.NSURLErrorTimedOut, -1001)
        self.assertEqual(Foundation.NSURLErrorUnsupportedURL, -1002)
        self.assertEqual(Foundation.NSURLErrorCannotFindHost, -1003)
        self.assertEqual(Foundation.NSURLErrorCannotConnectToHost, -1004)
        self.assertEqual(Foundation.NSURLErrorNetworkConnectionLost, -1005)
        self.assertEqual(Foundation.NSURLErrorDNSLookupFailed, -1006)
        self.assertEqual(Foundation.NSURLErrorHTTPTooManyRedirects, -1007)
        self.assertEqual(Foundation.NSURLErrorResourceUnavailable, -1008)
        self.assertEqual(Foundation.NSURLErrorNotConnectedToInternet, -1009)
        self.assertEqual(Foundation.NSURLErrorRedirectToNonExistentLocation, -1010)
        self.assertEqual(Foundation.NSURLErrorBadServerResponse, -1011)
        self.assertEqual(Foundation.NSURLErrorUserCancelledAuthentication, -1012)
        self.assertEqual(Foundation.NSURLErrorUserAuthenticationRequired, -1013)
        self.assertEqual(Foundation.NSURLErrorZeroByteResource, -1014)
        self.assertEqual(Foundation.NSURLErrorCannotDecodeRawData, -1015)
        self.assertEqual(Foundation.NSURLErrorCannotDecodeContentData, -1016)
        self.assertEqual(Foundation.NSURLErrorCannotParseResponse, -1017)
        self.assertEqual(Foundation.NSURLErrorFileDoesNotExist, -1100)
        self.assertEqual(Foundation.NSURLErrorFileIsDirectory, -1101)
        self.assertEqual(Foundation.NSURLErrorNoPermissionsToReadFile, -1102)
        self.assertEqual(Foundation.NSURLErrorSecureConnectionFailed, -1200)
        self.assertEqual(Foundation.NSURLErrorServerCertificateHasBadDate, -1201)
        self.assertEqual(Foundation.NSURLErrorServerCertificateUntrusted, -1202)
        self.assertEqual(Foundation.NSURLErrorServerCertificateHasUnknownRoot, -1203)
        self.assertEqual(Foundation.NSURLErrorServerCertificateNotYetValid, -1204)
        self.assertEqual(Foundation.NSURLErrorClientCertificateRejected, -1205)
        self.assertEqual(Foundation.NSURLErrorCannotLoadFromNetwork, -2000)
        self.assertEqual(Foundation.NSURLErrorCannotCreateFile, -3000)
        self.assertEqual(Foundation.NSURLErrorCannotOpenFile, -3001)
        self.assertEqual(Foundation.NSURLErrorCannotCloseFile, -3002)
        self.assertEqual(Foundation.NSURLErrorCannotWriteToFile, -3003)
        self.assertEqual(Foundation.NSURLErrorCannotRemoveFile, -3004)
        self.assertEqual(Foundation.NSURLErrorCannotMoveFile, -3005)
        self.assertEqual(Foundation.NSURLErrorDownloadDecodingFailedMidStream, -3006)
        self.assertEqual(Foundation.NSURLErrorDownloadDecodingFailedToComplete, -3007)

    @min_os_level("10.5")
    def testConstants10_5(self):
        self.assertEqual(Foundation.NSURLErrorDataLengthExceedsMaximum, -1103)

    @min_os_level("10.6")
    def testConstants10_6(self):
        self.assertIsInstance(Foundation.NSURLErrorFailingURLPeerTrustErrorKey, str)
        self.assertIsInstance(Foundation.NSURLErrorFailingURLErrorKey, str)
        self.assertIsInstance(Foundation.NSURLErrorFailingURLStringErrorKey, str)

        self.assertEqual(Foundation.NSURLErrorClientCertificateRequired, -1206)

    @min_os_level("10.7")
    def testConstants10_7(self):
        self.assertEqual(Foundation.NSURLErrorInternationalRoamingOff, -1018)
        self.assertEqual(Foundation.NSURLErrorCallIsActive, -1019)
        self.assertEqual(Foundation.NSURLErrorDataNotAllowed, -1020)
        self.assertEqual(Foundation.NSURLErrorRequestBodyStreamExhausted, -1021)

    @min_os_level("10.10")
    def testConstants10_10(self):
        self.assertIsInstance(
            Foundation.NSURLErrorBackgroundTaskCancelledReasonKey, str
        )

        self.assertEqual(
            Foundation.NSURLErrorCancelledReasonUserForceQuitApplication, 0
        )
        self.assertEqual(
            Foundation.NSURLErrorCancelledReasonBackgroundUpdatesDisabled, 1
        )
        self.assertEqual(
            Foundation.NSURLErrorCancelledReasonInsufficientSystemResources, 2
        )

        self.assertEqual(
            Foundation.NSURLErrorBackgroundSessionRequiresSharedContainer, -995
        )
        self.assertEqual(
            Foundation.NSURLErrorBackgroundSessionInUseByAnotherProcess, -996
        )
        self.assertEqual(Foundation.NSURLErrorBackgroundSessionWasDisconnected, -997)

    @min_os_level("10.11")
    def testConstants10_11(self):
        self.assertEqual(
            Foundation.NSURLErrorAppTransportSecurityRequiresSecureConnection, -1022
        )

    @min_os_level("10.12")
    def testConstants10_12(self):
        self.assertEqual(Foundation.NSURLErrorFileOutsideSafeArea, -1104)

    @min_os_level("10.15")
    def testConstants10_15(self):
        self.assertIsInstance(Foundation.NSURLErrorNetworkUnavailableReasonKey, str)

        self.assertEqual(Foundation.NSURLErrorNetworkUnavailableReasonCellular, 0)
        self.assertEqual(Foundation.NSURLErrorNetworkUnavailableReasonExpensive, 1)
        self.assertEqual(Foundation.NSURLErrorNetworkUnavailableReasonConstrained, 2)



import Foundation
import CoreFoundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSNumberFormatter(TestCase):
    def testConstants(self):
        self.assertEqual(
            Foundation.NSNumberFormatterNoStyle,
            CoreFoundation.kCFNumberFormatterNoStyle,
        )
        self.assertEqual(
            Foundation.NSNumberFormatterDecimalStyle,
            CoreFoundation.kCFNumberFormatterDecimalStyle,
        )
        self.assertEqual(
            Foundation.NSNumberFormatterCurrencyStyle,
            CoreFoundation.kCFNumberFormatterCurrencyStyle,
        )
        self.assertEqual(
            Foundation.NSNumberFormatterPercentStyle,
            CoreFoundation.kCFNumberFormatterPercentStyle,
        )
        self.assertEqual(
            Foundation.NSNumberFormatterScientificStyle,
            CoreFoundation.kCFNumberFormatterScientificStyle,
        )
        self.assertEqual(
            Foundation.NSNumberFormatterSpellOutStyle,
            CoreFoundation.kCFNumberFormatterSpellOutStyle,
        )

        self.assertEqual(Foundation.NSNumberFormatterBehaviorDefault, 0)
        self.assertEqual(Foundation.NSNumberFormatterBehavior10_0, 1000)
        self.assertEqual(Foundation.NSNumberFormatterBehavior10_4, 1040)

        self.assertEqual(
            Foundation.NSNumberFormatterPadBeforePrefix,
            CoreFoundation.kCFNumberFormatterPadBeforePrefix,
        )
        self.assertEqual(
            Foundation.NSNumberFormatterPadAfterPrefix,
            CoreFoundation.kCFNumberFormatterPadAfterPrefix,
        )
        self.assertEqual(
            Foundation.NSNumberFormatterPadBeforeSuffix,
            CoreFoundation.kCFNumberFormatterPadBeforeSuffix,
        )
        self.assertEqual(
            Foundation.NSNumberFormatterPadAfterSuffix,
            CoreFoundation.kCFNumberFormatterPadAfterSuffix,
        )

        self.assertEqual(
            Foundation.NSNumberFormatterRoundCeiling,
            CoreFoundation.kCFNumberFormatterRoundCeiling,
        )
        self.assertEqual(
            Foundation.NSNumberFormatterRoundFloor,
            CoreFoundation.kCFNumberFormatterRoundFloor,
        )
        self.assertEqual(
            Foundation.NSNumberFormatterRoundDown,
            CoreFoundation.kCFNumberFormatterRoundDown,
        )
        self.assertEqual(
            Foundation.NSNumberFormatterRoundUp,
            CoreFoundation.kCFNumberFormatterRoundUp,
        )
        self.assertEqual(
            Foundation.NSNumberFormatterRoundHalfEven,
            CoreFoundation.kCFNumberFormatterRoundHalfEven,
        )
        self.assertEqual(
            Foundation.NSNumberFormatterRoundHalfDown,
            CoreFoundation.kCFNumberFormatterRoundHalfDown,
        )
        self.assertEqual(
            Foundation.NSNumberFormatterRoundHalfUp,
            CoreFoundation.kCFNumberFormatterRoundHalfUp,
        )

    @min_os_level("10.11")
    def testConstants10_11(self):
        self.assertEqual(
            Foundation.NSNumberFormatterOrdinalStyle,
            CoreFoundation.kCFNumberFormatterOrdinalStyle,
        )
        self.assertEqual(
            Foundation.NSNumberFormatterCurrencyISOCodeStyle,
            CoreFoundation.kCFNumberFormatterCurrencyISOCodeStyle,
        )
        self.assertEqual(
            Foundation.NSNumberFormatterCurrencyPluralStyle,
            CoreFoundation.kCFNumberFormatterCurrencyPluralStyle,
        )
        self.assertEqual(
            Foundation.NSNumberFormatterCurrencyAccountingStyle,
            CoreFoundation.kCFNumberFormatterCurrencyAccountingStyle,
        )

    def testOutput(self):
        self.assertResultIsBOOL(
            Foundation.NSNumberFormatter.getObjectValue_forString_range_error_
        )
        self.assertArgIsOut(
            Foundation.NSNumberFormatter.getObjectValue_forString_range_error_, 0
        )
        self.assertArgIsInOut(
            Foundation.NSNumberFormatter.getObjectValue_forString_range_error_, 2
        )
        self.assertArgIsOut(
            Foundation.NSNumberFormatter.getObjectValue_forString_range_error_, 3
        )

        self.assertResultIsBOOL(Foundation.NSNumberFormatter.generatesDecimalNumbers)
        self.assertArgIsBOOL(
            Foundation.NSNumberFormatter.setGeneratesDecimalNumbers_, 0
        )
        self.assertResultIsBOOL(Foundation.NSNumberFormatter.allowsFloats)
        self.assertArgIsBOOL(Foundation.NSNumberFormatter.setAllowsFloats_, 0)
        self.assertResultIsBOOL(
            Foundation.NSNumberFormatter.alwaysShowsDecimalSeparator
        )
        self.assertArgIsBOOL(
            Foundation.NSNumberFormatter.setAlwaysShowsDecimalSeparator_, 0
        )
        self.assertResultIsBOOL(Foundation.NSNumberFormatter.usesGroupingSeparator)
        self.assertArgIsBOOL(Foundation.NSNumberFormatter.setUsesGroupingSeparator_, 0)
        self.assertResultIsBOOL(Foundation.NSNumberFormatter.isLenient)
        self.assertArgIsBOOL(Foundation.NSNumberFormatter.setLenient_, 0)
        self.assertResultIsBOOL(Foundation.NSNumberFormatter.usesSignificantDigits)
        self.assertArgIsBOOL(Foundation.NSNumberFormatter.setUsesSignificantDigits_, 0)
        self.assertResultIsBOOL(
            Foundation.NSNumberFormatter.isPartialStringValidationEnabled
        )
        self.assertArgIsBOOL(
            Foundation.NSNumberFormatter.setPartialStringValidationEnabled_, 0
        )
        self.assertResultIsBOOL(Foundation.NSNumberFormatter.hasThousandSeparators)
        self.assertArgIsBOOL(Foundation.NSNumberFormatter.setHasThousandSeparators_, 0)
        self.assertResultIsBOOL(Foundation.NSNumberFormatter.localizesFormat)
        self.assertArgIsBOOL(Foundation.NSNumberFormatter.setLocalizesFormat_, 0)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import objc


class TestNSPathControlHelper(AppKit.NSObject):
    def pathControl_shouldDragItem_withPasteboard_(self, pc, i, p):
        return 1

    def pathControl_shouldDragPathComponentCell_withPasteboard_(self, pc, c, pb):
        return 1

    def pathControl_validateDrop_(self, pc, dr):
        return 1

    def pathControl_acceptDrop_(self, pc, dr):
        return 1


class TestNSPathControl(TestCase):
    @min_os_level("10.10")
    def testMethods10_10(self):
        self.assertResultIsBOOL(AppKit.NSPathControl.isEditable)
        self.assertArgIsBOOL(AppKit.NSPathControl.setEditable_, 0)

    @min_os_level("10.5")
    def testMethods(self):
        m = AppKit.NSPathControl.setDoubleAction_.__metadata__()
        self.assertEqual(m["arguments"][2]["sel_of_type"], b"v@:@")

        self.assertArgIsBOOL(
            AppKit.NSPathControl.setDraggingSourceOperationMask_forLocal_, 1
        )

    @min_sdk_level("10.6")
    def testProtocolObjects(self):
        objc.protocolNamed("NSPathControlDelegate")

    @min_os_level("10.5")
    def testProtocols(self):

        self.assertResultIsBOOL(
            TestNSPathControlHelper.pathControl_shouldDragItem_withPasteboard_
        )
        self.assertResultIsBOOL(
            TestNSPathControlHelper.pathControl_shouldDragPathComponentCell_withPasteboard_
        )
        self.assertResultHasType(
            TestNSPathControlHelper.pathControl_validateDrop_, objc._C_NSUInteger
        )
        self.assertResultIsBOOL(TestNSPathControlHelper.pathControl_acceptDrop_)



import sys

import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level, onlyIf
import objc

try:
    import Quartz
except ImportError:
    Quartz = None


class TestNSWindowHelper(AppKit.NSObject):
    def window_willUseFullScreenContentSize_(self, a, b):
        pass

    def window_willUseFullScreenPresentationOptions_(self, a, b):
        pass

    def window_startCustomAnimationToEnterFullScreenWithDuration_(self, a, b):
        pass

    def window_startCustomAnimationToExitFullScreenWithDuration_(self, a, b):
        pass

    def window_willResizeForVersionBrowserWithMaxPreferredSize_maxAllowedSize_(
        self, a, b, c
    ):
        pass

    def windowShouldClose_(self, w):
        return 1

    def windowWillResize_toSize_(self, w, a):
        return 1

    def windowWillUseStandardFrame_defaultFrame_(self, w, a):
        return 1

    def windowShouldZoom_toFrame_(self, w, a):
        return 1

    def window_willPositionSheet_usingRect_(self, w, a, b):
        return 1

    def window_shouldPopUpDocumentPathMenu_(self, w, a):
        return 1

    def window_shouldDragDocumentWithEvent_from_withPasteboard_(self, w, a, b, c):
        return 1

    def window_startCustomAnimationToEnterFullScreenOnScreen_withDuration_(
        self, w, a, d
    ):
        pass


class TestNSWindow(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSEventDurationForever, sys.float_info.max)
        self.assertEqual(AppKit.NSAppKitVersionNumberWithCustomSheetPosition, 686.0)
        self.assertEqual(
            AppKit.NSAppKitVersionNumberWithDeferredWindowDisplaySupport, 1019.0
        )

        self.assertEqual(AppKit.NSBorderlessWindowMask, 0)
        self.assertEqual(AppKit.NSTitledWindowMask, 1 << 0)
        self.assertEqual(AppKit.NSClosableWindowMask, 1 << 1)
        self.assertEqual(AppKit.NSMiniaturizableWindowMask, 1 << 2)
        self.assertEqual(AppKit.NSResizableWindowMask, 1 << 3)

        self.assertEqual(AppKit.NSWindowStyleMaskBorderless, 0)
        self.assertEqual(AppKit.NSWindowStyleMaskTitled, 1 << 0)
        self.assertEqual(AppKit.NSWindowStyleMaskClosable, 1 << 1)
        self.assertEqual(AppKit.NSWindowStyleMaskMiniaturizable, 1 << 2)
        self.assertEqual(AppKit.NSWindowStyleMaskResizable, 1 << 3)
        self.assertEqual(AppKit.NSWindowStyleMaskTexturedBackground, 1 << 8)
        self.assertEqual(AppKit.NSWindowStyleMaskUnifiedTitleAndToolbar, 1 << 12)
        self.assertEqual(AppKit.NSWindowStyleMaskFullScreen, 1 << 14)
        self.assertEqual(AppKit.NSWindowStyleMaskFullSizeContentView, 1 << 15)
        self.assertEqual(AppKit.NSWindowStyleMaskUtilityWindow, 1 << 4)
        self.assertEqual(AppKit.NSWindowStyleMaskDocModalWindow, 1 << 6)
        self.assertEqual(AppKit.NSWindowStyleMaskNonactivatingPanel, 1 << 7)
        self.assertEqual(AppKit.NSWindowStyleMaskHUDWindow, 1 << 13)
        self.assertEqual(AppKit.NSWindowCollectionBehaviorFullScreenNone, 1 << 9)

        self.assertEqual(AppKit.NSWindowUserTabbingPreferenceManual, 0)
        self.assertEqual(AppKit.NSWindowUserTabbingPreferenceAlways, 1)
        self.assertEqual(AppKit.NSWindowUserTabbingPreferenceInFullScreen, 2)

        self.assertEqual(AppKit.NSWindowTabbingModeAutomatic, 0)
        self.assertEqual(AppKit.NSWindowTabbingModePreferred, 1)
        self.assertEqual(AppKit.NSWindowTabbingModeDisallowed, 2)

        self.assertEqual(AppKit.NSTexturedBackgroundWindowMask, 1 << 8)

        self.assertEqual(AppKit.NSUnscaledWindowMask, 1 << 11)

        self.assertEqual(AppKit.NSUnifiedTitleAndToolbarWindowMask, 1 << 12)

        self.assertEqual(AppKit.NSDisplayWindowRunLoopOrdering, 600_000)
        self.assertEqual(AppKit.NSResetCursorRectsRunLoopOrdering, 700_000)

        self.assertEqual(AppKit.NSWindowSharingNone, 0)
        self.assertEqual(AppKit.NSWindowSharingReadOnly, 1)
        self.assertEqual(AppKit.NSWindowSharingReadWrite, 2)

        self.assertEqual(AppKit.NSWindowBackingLocationDefault, 0)
        self.assertEqual(AppKit.NSWindowBackingLocationVideoMemory, 1)
        self.assertEqual(AppKit.NSWindowBackingLocationMainMemory, 2)

        self.assertEqual(AppKit.NSWindowCollectionBehaviorDefault, 0)
        self.assertEqual(AppKit.NSWindowCollectionBehaviorCanJoinAllSpaces, 1 << 0)
        self.assertEqual(AppKit.NSWindowCollectionBehaviorMoveToActiveSpace, 1 << 1)

        self.assertEqual(AppKit.NSDirectSelection, 0)
        self.assertEqual(AppKit.NSSelectingNext, 1)
        self.assertEqual(AppKit.NSSelectingPrevious, 2)

        self.assertEqual(AppKit.NSWindowCloseButton, 0)
        self.assertEqual(AppKit.NSWindowMiniaturizeButton, 1)
        self.assertEqual(AppKit.NSWindowZoomButton, 2)
        self.assertEqual(AppKit.NSWindowToolbarButton, 3)
        self.assertEqual(AppKit.NSWindowDocumentIconButton, 4)

        self.assertIsInstance(AppKit.NSWindowDidBecomeKeyNotification, str)
        self.assertIsInstance(AppKit.NSWindowDidBecomeMainNotification, str)
        self.assertIsInstance(AppKit.NSWindowDidChangeScreenNotification, str)
        self.assertIsInstance(AppKit.NSWindowDidDeminiaturizeNotification, str)
        self.assertIsInstance(AppKit.NSWindowDidExposeNotification, str)
        self.assertIsInstance(AppKit.NSWindowDidMiniaturizeNotification, str)
        self.assertIsInstance(AppKit.NSWindowDidMoveNotification, str)
        self.assertIsInstance(AppKit.NSWindowDidResignKeyNotification, str)
        self.assertIsInstance(AppKit.NSWindowDidResignMainNotification, str)
        self.assertIsInstance(AppKit.NSWindowDidResizeNotification, str)
        self.assertIsInstance(AppKit.NSWindowDidUpdateNotification, str)
        self.assertIsInstance(AppKit.NSWindowWillCloseNotification, str)
        self.assertIsInstance(AppKit.NSWindowWillMiniaturizeNotification, str)
        self.assertIsInstance(AppKit.NSWindowWillMoveNotification, str)
        self.assertIsInstance(AppKit.NSWindowWillBeginSheetNotification, str)
        self.assertIsInstance(AppKit.NSWindowDidEndSheetNotification, str)
        self.assertIsInstance(AppKit.NSWindowDidChangeScreenProfileNotification, str)

        self.assertEqual(AppKit.NSWindowToolbarStyleAutomatic, 0)
        self.assertEqual(AppKit.NSWindowToolbarStyleExpanded, 1)
        self.assertEqual(AppKit.NSWindowToolbarStylePreference, 2)
        self.assertEqual(AppKit.NSWindowToolbarStyleUnified, 3)
        self.assertEqual(AppKit.NSWindowToolbarStyleUnifiedCompact, 4)

        self.assertEqual(AppKit.NSTitlebarSeparatorStyleAutomatic, 0)
        self.assertEqual(AppKit.NSTitlebarSeparatorStyleNone, 1)
        self.assertEqual(AppKit.NSTitlebarSeparatorStyleLine, 2)
        self.assertEqual(AppKit.NSTitlebarSeparatorStyleShadow, 3)

    @min_os_level("10.6")
    def testConstants10_6(self):
        self.assertEqual(
            AppKit.NSAppKitVersionNumberWithDeferredWindowDisplaySupport, 1019.0
        )

        self.assertEqual(AppKit.NSWindowCollectionBehaviorManaged, 1 << 2)
        self.assertEqual(AppKit.NSWindowCollectionBehaviorTransient, 1 << 3)
        self.assertEqual(AppKit.NSWindowCollectionBehaviorStationary, 1 << 4)
        self.assertEqual(AppKit.NSWindowCollectionBehaviorParticipatesInCycle, 1 << 5)
        self.assertEqual(AppKit.NSWindowCollectionBehaviorIgnoresCycle, 1 << 6)
        self.assertEqual(AppKit.NSWindowNumberListAllApplications, 1 << 0)
        self.assertEqual(AppKit.NSWindowNumberListAllSpaces, 1 << 4)

        self.assertIsInstance(AppKit.NSWindowWillStartLiveResizeNotification, str)
        self.assertIsInstance(AppKit.NSWindowDidEndLiveResizeNotification, str)

    @min_os_level("10.7")
    def testConstants10_7(self):
        self.assertEqual(AppKit.NSFullScreenWindowMask, 1 << 14)
        self.assertEqual(AppKit.NSWindowCollectionBehaviorFullScreenPrimary, 1 << 7)
        self.assertEqual(AppKit.NSWindowCollectionBehaviorFullScreenAuxiliary, 1 << 8)
        self.assertEqual(AppKit.NSWindowAnimationBehaviorDefault, 0)
        self.assertEqual(AppKit.NSWindowAnimationBehaviorNone, 2)
        self.assertEqual(AppKit.NSWindowAnimationBehaviorDocumentWindow, 3)
        self.assertEqual(AppKit.NSWindowAnimationBehaviorUtilityWindow, 4)
        self.assertEqual(AppKit.NSWindowAnimationBehaviorAlertPanel, 5)
        self.assertEqual(AppKit.NSWindowDocumentVersionsButton, 6)
        self.assertEqual(AppKit.NSWindowFullScreenButton, 7)

        self.assertIsInstance(
            AppKit.NSWindowDidChangeBackingPropertiesNotification, str
        )
        self.assertIsInstance(AppKit.NSBackingPropertyOldScaleFactorKey, str)
        self.assertIsInstance(AppKit.NSBackingPropertyOldColorSpaceKey, str)
        self.assertIsInstance(AppKit.NSWindowWillEnterFullScreenNotification, str)
        self.assertIsInstance(AppKit.NSWindowDidEnterFullScreenNotification, str)
        self.assertIsInstance(AppKit.NSWindowWillExitFullScreenNotification, str)
        self.assertIsInstance(AppKit.NSWindowDidExitFullScreenNotification, str)
        self.assertIsInstance(AppKit.NSWindowWillEnterVersionBrowserNotification, str)
        self.assertIsInstance(AppKit.NSWindowDidEnterVersionBrowserNotification, str)
        self.assertIsInstance(AppKit.NSWindowWillExitVersionBrowserNotification, str)
        self.assertIsInstance(AppKit.NSWindowDidExitVersionBrowserNotification, str)

    @min_os_level("10.9")
    def testConstants10_9(self):
        self.assertEqual(AppKit.NSModalResponseOK, 1)
        self.assertEqual(AppKit.NSModalResponseCancel, 0)

        self.assertEqual(AppKit.NSWindowOcclusionStateVisible, 2)

        self.assertIsInstance(AppKit.NSWindowDidChangeOcclusionStateNotification, str)

    @min_os_level("10.10")
    def testConstants10_10(self):
        self.assertEqual(AppKit.NSFullSizeContentViewWindowMask, 1 << 15)

        self.assertEqual(AppKit.NSWindowTitleVisible, 0)
        self.assertEqual(AppKit.NSWindowTitleHidden, 1)

    @min_os_level("10.11")
    def testConstants10_11(self):
        self.assertEqual(
            AppKit.NSWindowCollectionBehaviorFullScreenAllowsTiling, 1 << 11
        )
        self.assertEqual(
            AppKit.NSWindowCollectionBehaviorFullScreenDisallowsTiling, 1 << 12
        )

    @onlyIf(Quartz is not None)
    def testMagicConstants(self):
        self.assertEqual(AppKit.NSNormalWindowLevel, Quartz.kCGNormalWindowLevel)
        self.assertEqual(AppKit.NSFloatingWindowLevel, Quartz.kCGFloatingWindowLevel)
        self.assertEqual(AppKit.NSSubmenuWindowLevel, Quartz.kCGTornOffMenuWindowLevel)
        self.assertEqual(
            AppKit.NSTornOffMenuWindowLevel, Quartz.kCGTornOffMenuWindowLevel
        )
        self.assertEqual(AppKit.NSMainMenuWindowLevel, Quartz.kCGMainMenuWindowLevel)
        self.assertEqual(AppKit.NSStatusWindowLevel, Quartz.kCGStatusWindowLevel)
        self.assertEqual(AppKit.NSDockWindowLevel, Quartz.kCGDockWindowLevel)
        self.assertEqual(
            AppKit.NSModalPanelWindowLevel, Quartz.kCGModalPanelWindowLevel
        )
        self.assertEqual(AppKit.NSPopUpMenuWindowLevel, Quartz.kCGPopUpMenuWindowLevel)
        self.assertEqual(
            AppKit.NSScreenSaverWindowLevel, Quartz.kCGScreenSaverWindowLevel
        )

    def testMethods(self):
        self.assertArgIsBOOL(
            AppKit.NSWindow.nextEventMatchingMask_untilDate_inMode_dequeue_, 3
        )
        self.assertArgIsBOOL(
            AppKit.NSWindow.initWithContentRect_styleMask_backing_defer_, 3
        )
        self.assertArgIsBOOL(
            AppKit.NSWindow.initWithContentRect_styleMask_backing_defer_screen_, 3
        )
        self.assertArgIsBOOL(AppKit.NSWindow.setExcludedFromWindowsMenu_, 0)
        self.assertResultIsBOOL(AppKit.NSWindow.isExcludedFromWindowsMenu)
        self.assertArgIsBOOL(AppKit.NSWindow.fieldEditor_forObject_, 0)
        self.assertArgIsBOOL(AppKit.NSWindow.setFrame_display_, 1)
        self.assertArgIsBOOL(AppKit.NSWindow.setFrame_display_animate_, 1)
        self.assertArgIsBOOL(AppKit.NSWindow.setFrame_display_animate_, 2)
        self.assertArgIsBOOL(AppKit.NSWindow.setShowsResizeIndicator_, 0)
        self.assertResultIsBOOL(AppKit.NSWindow.showsResizeIndicator)
        self.assertArgIsBOOL(AppKit.NSWindow.useOptimizedDrawing_, 0)
        self.assertResultIsBOOL(AppKit.NSWindow.isFlushWindowDisabled)
        self.assertResultIsBOOL(AppKit.NSWindow.viewsNeedDisplay)
        self.assertArgIsBOOL(AppKit.NSWindow.setViewsNeedDisplay_, 0)
        self.assertResultIsBOOL(AppKit.NSWindow.isAutodisplay)
        self.assertArgIsBOOL(AppKit.NSWindow.setAutodisplay_, 0)
        self.assertResultIsBOOL(AppKit.NSWindow.preservesContentDuringLiveResize)
        self.assertArgIsBOOL(AppKit.NSWindow.setPreservesContentDuringLiveResize_, 0)
        self.assertResultIsBOOL(AppKit.NSWindow.makeFirstResponder_)
        self.assertArgIsBOOL(AppKit.NSWindow.setReleasedWhenClosed_, 0)
        self.assertResultIsBOOL(AppKit.NSWindow.isReleasedWhenClosed)
        self.assertResultIsBOOL(AppKit.NSWindow.isZoomed)
        self.assertResultIsBOOL(AppKit.NSWindow.isMiniaturized)
        self.assertResultIsBOOL(AppKit.NSWindow.tryToPerform_with_)
        self.assertResultIsBOOL(AppKit.NSWindow.isMovableByWindowBackground)
        self.assertArgIsBOOL(AppKit.NSWindow.setMovableByWindowBackground_, 0)
        self.assertResultIsBOOL(AppKit.NSWindow.hidesOnDeactivate)
        self.assertArgIsBOOL(AppKit.NSWindow.setHidesOnDeactivate_, 0)
        self.assertResultIsBOOL(AppKit.NSWindow.canHide)
        self.assertArgIsBOOL(AppKit.NSWindow.setCanHide_, 0)
        self.assertArgIsBOOL(AppKit.NSWindow.setDocumentEdited_, 0)
        self.assertResultIsBOOL(AppKit.NSWindow.isDocumentEdited)
        self.assertResultIsBOOL(AppKit.NSWindow.isVisible)
        self.assertResultIsBOOL(AppKit.NSWindow.isKeyWindow)
        self.assertResultIsBOOL(AppKit.NSWindow.isMainWindow)
        self.assertResultIsBOOL(AppKit.NSWindow.canBecomeKeyWindow)
        self.assertResultIsBOOL(AppKit.NSWindow.canBecomeMainWindow)
        self.assertResultIsBOOL(AppKit.NSWindow.worksWhenModal)
        self.assertResultIsBOOL(AppKit.NSWindow.isOneShot)
        self.assertArgIsBOOL(AppKit.NSWindow.setOneShot_, 0)
        self.assertResultIsBOOL(AppKit.NSWindow.areCursorRectsEnabled)
        self.assertArgIsBOOL(
            AppKit.NSWindow.setAllowsToolTipsWhenApplicationIsInactive_, 0
        )
        self.assertResultIsBOOL(AppKit.NSWindow.allowsToolTipsWhenApplicationIsInactive)
        self.assertArgIsBOOL(AppKit.NSWindow.setDynamicDepthLimit_, 0)
        self.assertResultIsBOOL(AppKit.NSWindow.hasDynamicDepthLimit)
        self.assertArgIsBOOL(AppKit.NSWindow.setHasShadow_, 0)
        self.assertResultIsBOOL(AppKit.NSWindow.hasShadow)
        self.assertResultIsBOOL(AppKit.NSWindow.canStoreColor)
        self.assertArgIsBOOL(AppKit.NSWindow.setOpaque_, 0)
        self.assertResultIsBOOL(AppKit.NSWindow.isOpaque)
        self.assertArgIsBOOL(AppKit.NSWindow.setDisplaysWhenScreenProfileChanges_, 0)
        self.assertResultIsBOOL(AppKit.NSWindow.displaysWhenScreenProfileChanges)
        self.assertResultIsBOOL(AppKit.NSWindow.setFrameUsingName_force_)
        self.assertArgIsBOOL(AppKit.NSWindow.setFrameUsingName_force_, 1)
        self.assertResultIsBOOL(AppKit.NSWindow.setFrameUsingName_)
        self.assertResultIsBOOL(AppKit.NSWindow.setFrameAutosaveName_)
        self.assertArgIsBOOL(AppKit.NSWindow.postEvent_atStart_, 1)
        self.assertResultIsBOOL(AppKit.NSWindow.acceptsMouseMovedEvents)
        self.assertArgIsBOOL(AppKit.NSWindow.setAcceptsMouseMovedEvents_, 0)
        self.assertArgIsBOOL(AppKit.NSWindow.setIgnoresMouseEvents_, 0)
        self.assertResultIsBOOL(AppKit.NSWindow.ignoresMouseEvents)
        self.assertResultIsBOOL(AppKit.NSWindow.isSheet)
        self.assertArgIsBOOL(AppKit.NSWindow.setAutorecalculatesKeyViewLoop_, 0)
        self.assertResultIsBOOL(AppKit.NSWindow.autorecalculatesKeyViewLoop)
        self.assertArgIsBOOL(AppKit.NSWindow.setShowsToolbarButton_, 0)
        self.assertResultIsBOOL(AppKit.NSWindow.showsToolbarButton)
        self.assertArgIsBOOL(
            AppKit.NSWindow.dragImage_at_offset_event_pasteboard_source_slideBack_, 6
        )

    @min_os_level("10.5")
    def testMethods10_5(self):
        self.assertResultIsBOOL(
            AppKit.NSWindow.autorecalculatesContentBorderThicknessForEdge_
        )
        self.assertArgIsBOOL(
            AppKit.NSWindow.setAutorecalculatesContentBorderThickness_forEdge_, 0
        )
        self.assertArgIsBOOL(AppKit.NSWindow.setCanBecomeVisibleWithoutLogin_, 0)
        self.assertResultIsBOOL(AppKit.NSWindow.canBecomeVisibleWithoutLogin)
        self.assertResultIsBOOL(AppKit.NSWindow.canBeVisibleOnAllSpaces)
        self.assertArgIsBOOL(AppKit.NSWindow.setCanBeVisibleOnAllSpaces_, 0)

    @min_os_level("10.6")
    def testMethods10_6(self):
        self.assertResultIsBOOL(AppKit.NSWindow.inLiveResize)
        self.assertResultIsBOOL(AppKit.NSWindow.isOnActiveSpace)
        self.assertResultIsBOOL(AppKit.NSWindow.isMovable)
        self.assertArgIsBOOL(AppKit.NSWindow.setMovable_, 0)
        self.assertResultIsBOOL(AppKit.NSWindow.preventsApplicationTerminationWhenModal)
        self.assertArgIsBOOL(
            AppKit.NSWindow.setPreventsApplicationTerminationWhenModal_, 0
        )
        self.assertResultIsBOOL(AppKit.NSWindow.allowsConcurrentViewDrawing)
        self.assertArgIsBOOL(AppKit.NSWindow.setAllowsConcurrentViewDrawing_, 0)

        self.assertArgHasType(
            AppKit.NSWindow.windowNumberAtPoint_belowWindowWithWindowNumber_,
            0,
            AppKit.NSPoint.__typestr__,
        )

    @min_os_level("10.9")
    def testMethods10_9(self):
        self.assertArgIsBlock(
            AppKit.NSWindow.beginSheet_completionHandler_, 1, b"v" + objc._C_NSInteger
        )
        self.assertArgIsBlock(
            AppKit.NSWindow.beginCriticalSheet_completionHandler_,
            1,
            b"v" + objc._C_NSInteger,
        )

    @min_os_level("10.10")
    def testMethods10_10(self):
        self.assertResultIsBOOL(AppKit.NSWindow.titlebarAppearsTransparent)
        self.assertArgIsBOOL(AppKit.NSWindow.setTitlebarAppearsTransparent_, 0)

        self.assertArgIsBlock(
            AppKit.NSWindow.trackEventsMatchingMask_timeout_mode_handler_, 3, b"v@o^Z"
        )

    @min_os_level("10.12")
    def testMethods10_12(self):
        self.assertResultIsBOOL(AppKit.NSWindow.canRepresentDisplayGamut_)

        self.assertResultIsBOOL(AppKit.NSWindow.allowsAutomaticWindowTabbing)
        self.assertArgIsBOOL(AppKit.NSWindow.setAllowsAutomaticWindowTabbing_, 0)

    @min_sdk_level("10.6")
    def testProtocolObjects(self):
        objc.protocolNamed("NSWindowDelegate")

    def testProtocols(self):
        self.assertResultIsBOOL(TestNSWindowHelper.windowShouldClose_)
        self.assertResultHasType(
            TestNSWindowHelper.windowWillResize_toSize_, AppKit.NSSize.__typestr__
        )
        self.assertArgHasType(
            TestNSWindowHelper.windowWillResize_toSize_, 1, AppKit.NSSize.__typestr__
        )
        self.assertResultHasType(
            TestNSWindowHelper.windowWillUseStandardFrame_defaultFrame_,
            AppKit.NSRect.__typestr__,
        )
        self.assertArgHasType(
            TestNSWindowHelper.windowWillUseStandardFrame_defaultFrame_,
            1,
            AppKit.NSRect.__typestr__,
        )
        self.assertResultIsBOOL(TestNSWindowHelper.windowShouldZoom_toFrame_)
        self.assertArgHasType(
            TestNSWindowHelper.windowShouldZoom_toFrame_, 1, AppKit.NSRect.__typestr__
        )
        self.assertResultHasType(
            TestNSWindowHelper.window_willPositionSheet_usingRect_,
            AppKit.NSRect.__typestr__,
        )
        self.assertArgHasType(
            TestNSWindowHelper.window_willPositionSheet_usingRect_,
            2,
            AppKit.NSRect.__typestr__,
        )
        self.assertResultIsBOOL(TestNSWindowHelper.window_shouldPopUpDocumentPathMenu_)
        self.assertResultIsBOOL(
            TestNSWindowHelper.window_shouldDragDocumentWithEvent_from_withPasteboard_
        )
        self.assertArgHasType(
            TestNSWindowHelper.window_shouldDragDocumentWithEvent_from_withPasteboard_,
            2,
            AppKit.NSPoint.__typestr__,
        )

    @min_os_level("10.7")
    def testProtocols10_7(self):
        self.assertArgHasType(
            TestNSWindowHelper.window_willUseFullScreenContentSize_,
            1,
            AppKit.NSSize.__typestr__,
        )
        self.assertArgHasType(
            TestNSWindowHelper.window_willUseFullScreenPresentationOptions_,
            1,
            objc._C_NSUInteger,
        )
        self.assertResultHasType(
            TestNSWindowHelper.window_willUseFullScreenPresentationOptions_,
            objc._C_NSUInteger,
        )
        self.assertArgHasType(
            TestNSWindowHelper.window_startCustomAnimationToEnterFullScreenWithDuration_,
            1,
            objc._C_DBL,
        )
        self.assertArgHasType(
            TestNSWindowHelper.window_startCustomAnimationToExitFullScreenWithDuration_,
            1,
            objc._C_DBL,
        )
        self.assertResultHasType(
            TestNSWindowHelper.window_willResizeForVersionBrowserWithMaxPreferredSize_maxAllowedSize_,  # noqa: B950
            AppKit.NSSize.__typestr__,
        )
        self.assertArgHasType(
            TestNSWindowHelper.window_willResizeForVersionBrowserWithMaxPreferredSize_maxAllowedSize_,  # noqa: B950
            1,
            AppKit.NSSize.__typestr__,
        )
        self.assertArgHasType(
            TestNSWindowHelper.window_willResizeForVersionBrowserWithMaxPreferredSize_maxAllowedSize_,  # noqa: B950
            2,
            AppKit.NSSize.__typestr__,
        )

    @min_os_level("10.9")
    def testProtocols10_9(self):
        self.assertArgHasType(
            TestNSWindowHelper.window_startCustomAnimationToEnterFullScreenOnScreen_withDuration_,  # noqa: B950
            2,
            objc._C_DBL,
        )



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import objc


class TestNSTouchBar(TestCase):
    @min_os_level("10.12")
    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSTouchBar.isVisible)
        # self.assertArgIsBOOL(AppKit.NSTouchBar.setVisible_, 0)

        self.assertResultIsBOOL(
            AppKit.NSApplication.isAutomaticCustomizeTouchBarMenuItemEnabled
        )
        self.assertArgIsBOOL(
            AppKit.NSApplication.setAutomaticCustomizeTouchBarMenuItemEnabled_, 0
        )

    @min_os_level("10.15")
    def testMethods10_15(self):
        self.assertResultIsBOOL(
            AppKit.NSTouchBar.isAutomaticCustomizeTouchBarMenuItemEnabled
        )
        self.assertArgIsBOOL(
            AppKit.NSTouchBar.setAutomaticCustomizeTouchBarMenuItemEnabled_, 0
        )

    @min_sdk_level("10.12")
    def testProtocolObjects(self):
        objc.protocolNamed("NSTouchBarDelegate")
        objc.protocolNamed("NSTouchBarProvider")



import CoreFoundation
import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestDateFormatter(TestCase):
    def testTypeID(self):
        self.assertIsInstance(CoreFoundation.CFDateGetTypeID(), int)

    def testConstants(self):
        self.assertEqual(CoreFoundation.kCFDateFormatterNoStyle, 0)
        self.assertEqual(CoreFoundation.kCFDateFormatterShortStyle, 1)
        self.assertEqual(CoreFoundation.kCFDateFormatterMediumStyle, 2)
        self.assertEqual(CoreFoundation.kCFDateFormatterLongStyle, 3)
        self.assertEqual(CoreFoundation.kCFDateFormatterFullStyle, 4)
        self.assertIsInstance(CoreFoundation.kCFDateFormatterIsLenient, str)
        self.assertIsInstance(CoreFoundation.kCFDateFormatterTimeZone, str)
        self.assertIsInstance(CoreFoundation.kCFDateFormatterCalendarName, str)
        self.assertIsInstance(CoreFoundation.kCFDateFormatterDefaultFormat, str)
        self.assertIsInstance(CoreFoundation.kCFDateFormatterTwoDigitStartDate, str)
        self.assertIsInstance(CoreFoundation.kCFDateFormatterDefaultDate, str)
        self.assertIsInstance(CoreFoundation.kCFDateFormatterCalendar, str)
        self.assertIsInstance(CoreFoundation.kCFDateFormatterEraSymbols, str)
        self.assertIsInstance(CoreFoundation.kCFDateFormatterMonthSymbols, str)
        self.assertIsInstance(CoreFoundation.kCFDateFormatterShortMonthSymbols, str)
        self.assertIsInstance(CoreFoundation.kCFDateFormatterWeekdaySymbols, str)
        self.assertIsInstance(CoreFoundation.kCFDateFormatterShortWeekdaySymbols, str)
        self.assertIsInstance(CoreFoundation.kCFDateFormatterAMSymbol, str)
        self.assertIsInstance(CoreFoundation.kCFDateFormatterPMSymbol, str)
        self.assertIsInstance(CoreFoundation.kCFDateFormatterLongEraSymbols, str)
        self.assertIsInstance(CoreFoundation.kCFDateFormatterVeryShortMonthSymbols, str)
        self.assertIsInstance(
            CoreFoundation.kCFDateFormatterStandaloneMonthSymbols, str
        )
        self.assertIsInstance(
            CoreFoundation.kCFDateFormatterShortStandaloneMonthSymbols, str
        )
        self.assertIsInstance(
            CoreFoundation.kCFDateFormatterVeryShortStandaloneMonthSymbols, str
        )
        self.assertIsInstance(
            CoreFoundation.kCFDateFormatterVeryShortWeekdaySymbols, str
        )
        self.assertIsInstance(
            CoreFoundation.kCFDateFormatterStandaloneWeekdaySymbols, str
        )
        self.assertIsInstance(
            CoreFoundation.kCFDateFormatterShortStandaloneWeekdaySymbols, str
        )
        self.assertIsInstance(
            CoreFoundation.kCFDateFormatterVeryShortStandaloneWeekdaySymbols, str
        )
        self.assertIsInstance(CoreFoundation.kCFDateFormatterQuarterSymbols, str)
        self.assertIsInstance(CoreFoundation.kCFDateFormatterShortQuarterSymbols, str)
        self.assertIsInstance(
            CoreFoundation.kCFDateFormatterStandaloneQuarterSymbols, str
        )
        self.assertIsInstance(
            CoreFoundation.kCFDateFormatterShortStandaloneQuarterSymbols, str
        )
        self.assertIsInstance(CoreFoundation.kCFDateFormatterGregorianStartDate, str)

        self.assertEqual(CoreFoundation.kCFISO8601DateFormatWithYear, 1 << 0)
        self.assertEqual(CoreFoundation.kCFISO8601DateFormatWithMonth, 1 << 1)
        self.assertEqual(CoreFoundation.kCFISO8601DateFormatWithWeekOfYear, 1 << 2)
        self.assertEqual(CoreFoundation.kCFISO8601DateFormatWithDay, 1 << 4)
        self.assertEqual(CoreFoundation.kCFISO8601DateFormatWithTime, 1 << 5)
        self.assertEqual(CoreFoundation.kCFISO8601DateFormatWithTimeZone, 1 << 6)
        self.assertEqual(
            CoreFoundation.kCFISO8601DateFormatWithSpaceBetweenDateAndTime, 1 << 7
        )
        self.assertEqual(
            CoreFoundation.kCFISO8601DateFormatWithDashSeparatorInDate, 1 << 8
        )
        self.assertEqual(
            CoreFoundation.kCFISO8601DateFormatWithColonSeparatorInTime, 1 << 9
        )
        self.assertEqual(
            CoreFoundation.kCFISO8601DateFormatWithColonSeparatorInTimeZone, 1 << 10
        )
        self.assertEqual(
            CoreFoundation.kCFISO8601DateFormatWithFractionalSeconds, 1 << 11
        )
        self.assertEqual(
            CoreFoundation.kCFISO8601DateFormatWithFullDate,
            CoreFoundation.kCFISO8601DateFormatWithYear
            | CoreFoundation.kCFISO8601DateFormatWithMonth
            | CoreFoundation.kCFISO8601DateFormatWithDay
            | CoreFoundation.kCFISO8601DateFormatWithDashSeparatorInDate,
        )
        self.assertEqual(
            CoreFoundation.kCFISO8601DateFormatWithFullTime,
            CoreFoundation.kCFISO8601DateFormatWithTime
            | CoreFoundation.kCFISO8601DateFormatWithColonSeparatorInTime
            | CoreFoundation.kCFISO8601DateFormatWithTimeZone
            | CoreFoundation.kCFISO8601DateFormatWithColonSeparatorInTimeZone,
        )
        self.assertEqual(
            CoreFoundation.kCFISO8601DateFormatWithInternetDateTime,
            CoreFoundation.kCFISO8601DateFormatWithFullDate
            | CoreFoundation.kCFISO8601DateFormatWithFullTime,
        )

    @min_os_level("10.6")
    def testConstants10_6(self):
        self.assertIsInstance(
            CoreFoundation.kCFDateFormatterDoesRelativeDateFormattingKey, str
        )

    @min_os_level("10.6")
    def testFunction10_6(self):
        self.assertResultIsCFRetained(
            CoreFoundation.CFDateFormatterCreateDateFormatFromTemplate
        )
        r = CoreFoundation.CFDateFormatterCreateDateFormatFromTemplate(
            None, "%Y-%m-%d", 0, None
        )
        self.assertIsInstance(r, str)

    @min_os_level("10.12")
    def testFunctions10_12(self):
        self.assertResultIsCFRetained(
            CoreFoundation.CFDateFormatterCreateISO8601Formatter
        )

    def testTypes(self):
        self.assertIsCFType(CoreFoundation.CFDateFormatterRef)

        self.assertIsInstance(CoreFoundation.CFDateFormatterGetTypeID(), int)

    def testInspection(self):
        locale = CoreFoundation.CFLocaleCopyCurrent()
        self.assertIsInstance(locale, CoreFoundation.CFLocaleRef)

        date = CoreFoundation.CFDateCreate(
            None, CoreFoundation.CFAbsoluteTimeGetCurrent()
        )
        self.assertIsInstance(date, Foundation.NSDate)

        self.assertResultIsCFRetained(CoreFoundation.CFDateFormatterCreate)
        fmt = CoreFoundation.CFDateFormatterCreate(
            None,
            locale,
            CoreFoundation.kCFDateFormatterShortStyle,
            CoreFoundation.kCFDateFormatterLongStyle,
        )
        self.assertIsInstance(fmt, CoreFoundation.CFDateFormatterRef)
        v = CoreFoundation.CFDateFormatterGetLocale(fmt)
        self.assertEqual(
            CoreFoundation.CFLocaleGetIdentifier(locale),
            CoreFoundation.CFLocaleGetIdentifier(v),
        )

        v = CoreFoundation.CFDateFormatterGetDateStyle(fmt)
        self.assertEqual(v, CoreFoundation.kCFDateFormatterShortStyle)

        v = CoreFoundation.CFDateFormatterGetTimeStyle(fmt)
        self.assertEqual(v, CoreFoundation.kCFDateFormatterLongStyle)

        v = CoreFoundation.CFDateFormatterGetFormat(fmt)
        self.assertIsInstance(v, str)
        CoreFoundation.CFDateFormatterSetFormat(fmt, v[:-1])
        v2 = CoreFoundation.CFDateFormatterGetFormat(fmt)
        self.assertEqual(v[:-1], v2)

        v = CoreFoundation.CFDateFormatterCreateStringWithDate(None, fmt, date)
        self.assertIsInstance(v, str)
        v = CoreFoundation.CFDateFormatterCreateStringWithAbsoluteTime(
            None, fmt, CoreFoundation.CFAbsoluteTimeGetCurrent()
        )
        self.assertIsInstance(v, str)
        dt, rng = CoreFoundation.CFDateFormatterCreateDateFromString(
            None, fmt, v, (0, len(v))
        )
        self.assertIsInstance(dt, Foundation.NSDate)
        self.assertIsInstance(rng, CoreFoundation.CFRange)
        ok, rng, abstime = CoreFoundation.CFDateFormatterGetAbsoluteTimeFromString(
            fmt, v, (0, len(v)), None
        )
        self.assertIs(ok, True)
        self.assertIsInstance(rng, CoreFoundation.CFRange)
        self.assertIsInstance(abstime, float)
        self.assertResultIsCFRetained(CoreFoundation.CFDateFormatterCopyProperty)
        v = CoreFoundation.CFDateFormatterCopyProperty(
            fmt, CoreFoundation.kCFDateFormatterCalendarName
        )
        self.assertIsInstance(v, str)
        CoreFoundation.CFDateFormatterSetProperty(
            fmt, CoreFoundation.kCFDateFormatterCalendarName, "gregorian"
        )
        v = CoreFoundation.CFDateFormatterCopyProperty(
            fmt, CoreFoundation.kCFDateFormatterCalendarName
        )
        self.assertIsInstance(v, str)
        self.assertEqual(v, "gregorian")
        v = CoreFoundation.CFDateFormatterCopyProperty(
            fmt, CoreFoundation.kCFDateFormatterIsLenient
        )
        self.assertTrue(v is True or v is False)
        CoreFoundation.CFDateFormatterSetProperty(
            fmt, CoreFoundation.kCFDateFormatterIsLenient, True
        )
        v2 = CoreFoundation.CFDateFormatterCopyProperty(
            fmt, CoreFoundation.kCFDateFormatterIsLenient
        )
        self.assertIs(v2, True)



import AppKit  # noqa: F401
from PyObjCTools.TestSupport import TestCase, min_sdk_level
import objc


class TestNSSharingServicePickerToolbarItem(TestCase):
    @min_sdk_level("10.15")
    def test_protocols(self):
        objc.protocolNamed("NSSharingServicePickerToolbarItemDelegate")



import Foundation
from PyObjCTools.TestSupport import TestCase


class TestNSNotificationQueue(TestCase):
    def testConstants(self):
        self.assertEqual(Foundation.NSPostWhenIdle, 1)
        self.assertEqual(Foundation.NSPostASAP, 2)
        self.assertEqual(Foundation.NSPostNow, 3)

        self.assertEqual(Foundation.NSNotificationNoCoalescing, 0)
        self.assertEqual(Foundation.NSNotificationCoalescingOnName, 1)
        self.assertEqual(Foundation.NSNotificationCoalescingOnSender, 2)



import Foundation
import objc
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSLinguisticTagger(TestCase):
    @min_os_level("10.7")
    def testConstants(self):
        self.assertIsInstance(Foundation.NSLinguisticTagSchemeTokenType, str)
        self.assertIsInstance(Foundation.NSLinguisticTagSchemeLexicalClass, str)
        self.assertIsInstance(Foundation.NSLinguisticTagSchemeNameType, str)
        self.assertIsInstance(
            Foundation.NSLinguisticTagSchemeNameTypeOrLexicalClass, str
        )
        self.assertIsInstance(Foundation.NSLinguisticTagSchemeLemma, str)
        self.assertIsInstance(Foundation.NSLinguisticTagSchemeLanguage, str)
        self.assertIsInstance(Foundation.NSLinguisticTagSchemeScript, str)
        self.assertIsInstance(Foundation.NSLinguisticTagWord, str)
        self.assertIsInstance(Foundation.NSLinguisticTagPunctuation, str)
        self.assertIsInstance(Foundation.NSLinguisticTagWhitespace, str)
        self.assertIsInstance(Foundation.NSLinguisticTagOther, str)
        self.assertIsInstance(Foundation.NSLinguisticTagNoun, str)
        self.assertIsInstance(Foundation.NSLinguisticTagVerb, str)
        self.assertIsInstance(Foundation.NSLinguisticTagAdjective, str)
        self.assertIsInstance(Foundation.NSLinguisticTagAdverb, str)
        self.assertIsInstance(Foundation.NSLinguisticTagPronoun, str)
        self.assertIsInstance(Foundation.NSLinguisticTagDeterminer, str)
        self.assertIsInstance(Foundation.NSLinguisticTagParticle, str)
        self.assertIsInstance(Foundation.NSLinguisticTagPreposition, str)
        self.assertIsInstance(Foundation.NSLinguisticTagNumber, str)
        self.assertIsInstance(Foundation.NSLinguisticTagConjunction, str)
        self.assertIsInstance(Foundation.NSLinguisticTagInterjection, str)
        self.assertIsInstance(Foundation.NSLinguisticTagClassifier, str)
        self.assertIsInstance(Foundation.NSLinguisticTagIdiom, str)
        self.assertIsInstance(Foundation.NSLinguisticTagOtherWord, str)
        self.assertIsInstance(Foundation.NSLinguisticTagSentenceTerminator, str)
        self.assertIsInstance(Foundation.NSLinguisticTagOpenQuote, str)
        self.assertIsInstance(Foundation.NSLinguisticTagCloseQuote, str)
        self.assertIsInstance(Foundation.NSLinguisticTagOpenParenthesis, str)
        self.assertIsInstance(Foundation.NSLinguisticTagCloseParenthesis, str)
        self.assertIsInstance(Foundation.NSLinguisticTagWordJoiner, str)
        self.assertIsInstance(Foundation.NSLinguisticTagDash, str)
        self.assertIsInstance(Foundation.NSLinguisticTagOtherPunctuation, str)
        self.assertIsInstance(Foundation.NSLinguisticTagParagraphBreak, str)
        self.assertIsInstance(Foundation.NSLinguisticTagOtherWhitespace, str)
        self.assertIsInstance(Foundation.NSLinguisticTagPersonalName, str)
        self.assertIsInstance(Foundation.NSLinguisticTagPlaceName, str)
        self.assertIsInstance(Foundation.NSLinguisticTagOrganizationName, str)

        self.assertEqual(Foundation.NSLinguisticTaggerOmitWords, 1 << 0)
        self.assertEqual(Foundation.NSLinguisticTaggerOmitPunctuation, 1 << 1)
        self.assertEqual(Foundation.NSLinguisticTaggerOmitWhitespace, 1 << 2)
        self.assertEqual(Foundation.NSLinguisticTaggerOmitOther, 1 << 3)
        self.assertEqual(Foundation.NSLinguisticTaggerJoinNames, 1 << 4)

        self.assertEqual(Foundation.NSLinguisticTaggerUnitWord, 0)
        self.assertEqual(Foundation.NSLinguisticTaggerUnitSentence, 1)
        self.assertEqual(Foundation.NSLinguisticTaggerUnitParagraph, 2)
        self.assertEqual(Foundation.NSLinguisticTaggerUnitDocument, 3)

    @min_os_level("10.7")
    def testMethods(self):
        self.assertArgHasType(
            Foundation.NSLinguisticTagger.orthographyAtIndex_effectiveRange_,
            1,
            b"o^" + Foundation.NSRange.__typestr__,
        )

        self.assertArgIsBlock(
            Foundation.NSLinguisticTagger.enumerateTagsInRange_scheme_options_usingBlock_,
            3,
            b"v@"
            + Foundation.NSRange.__typestr__
            + Foundation.NSRange.__typestr__
            + b"o^"
            + objc._C_NSBOOL,
        )

        self.assertArgHasType(
            Foundation.NSLinguisticTagger.tagAtIndex_scheme_tokenRange_sentenceRange_,
            2,
            b"o^" + Foundation.NSRange.__typestr__,
        )
        self.assertArgHasType(
            Foundation.NSLinguisticTagger.tagAtIndex_scheme_tokenRange_sentenceRange_,
            3,
            b"o^" + Foundation.NSRange.__typestr__,
        )

        self.assertArgHasType(
            Foundation.NSLinguisticTagger.tagsInRange_scheme_options_tokenRanges_,
            3,
            b"o^@",
        )

        self.assertArgHasType(
            Foundation.NSLinguisticTagger.possibleTagsAtIndex_scheme_tokenRange_sentenceRange_scores_,  # noqa: B950
            2,
            b"o^" + Foundation.NSRange.__typestr__,
        )
        self.assertArgHasType(
            Foundation.NSLinguisticTagger.possibleTagsAtIndex_scheme_tokenRange_sentenceRange_scores_,  # noqa: B950
            3,
            b"o^" + Foundation.NSRange.__typestr__,
        )
        self.assertArgHasType(
            Foundation.NSLinguisticTagger.possibleTagsAtIndex_scheme_tokenRange_sentenceRange_scores_,  # noqa: B950
            4,
            b"o^@",
        )

        self.assertArgIsOut(
            Foundation.NSString.linguisticTagsInRange_scheme_options_orthography_tokenRanges_,  # noqa: B950
            4,
        )

        self.assertArgIsBlock(
            Foundation.NSString.enumerateLinguisticTagsInRange_scheme_options_orthography_usingBlock_,  # noqa: B950
            4,
            b"v@"
            + Foundation.NSRange.__typestr__
            + Foundation.NSRange.__typestr__
            + b"o^"
            + objc._C_NSBOOL,
        )

    @min_os_level("10.13")
    def testMethods10_13(self):
        self.assertArgHasType(
            Foundation.NSLinguisticTagger.enumerateTagsInRange_unit_scheme_options_usingBlock_,
            0,
            Foundation.NSRange.__typestr__,
        )
        self.assertArgIsBlock(
            Foundation.NSLinguisticTagger.enumerateTagsInRange_unit_scheme_options_usingBlock_,
            4,
            b"v@" + Foundation.NSRange.__typestr__ + b"o^Z",
        )
        self.assertArgHasType(
            Foundation.NSLinguisticTagger.tagAtIndex_unit_scheme_tokenRange_,
            3,
            b"o^" + Foundation.NSRange.__typestr__,
        )

        self.assertArgHasType(
            Foundation.NSLinguisticTagger.tagsInRange_unit_scheme_options_tokenRanges_,
            0,
            Foundation.NSRange.__typestr__,
        )
        self.assertArgIsOut(
            Foundation.NSLinguisticTagger.tagsInRange_unit_scheme_options_tokenRanges_,
            4,
        )
        self.assertArgIsOut(
            Foundation.NSLinguisticTagger.tagForString_atIndex_unit_scheme_orthography_tokenRange_,  # noqa: B950
            5,
        )
        self.assertArgHasType(
            Foundation.NSLinguisticTagger.tagsForString_range_unit_scheme_options_orthography_tokenRanges_,  # noqa: B950
            1,
            Foundation.NSRange.__typestr__,
        )
        self.assertArgIsOut(
            Foundation.NSLinguisticTagger.tagsForString_range_unit_scheme_options_orthography_tokenRanges_,  # noqa: B950
            6,
        )
        self.assertArgHasType(
            Foundation.NSLinguisticTagger.enumerateTagsForString_range_unit_scheme_options_orthography_usingBlock_,  # noqa: B950
            1,
            Foundation.NSRange.__typestr__,
        )
        self.assertArgIsBlock(
            Foundation.NSLinguisticTagger.enumerateTagsForString_range_unit_scheme_options_orthography_usingBlock_,  # noqa: B950
            6,
            b"v@" + Foundation.NSRange.__typestr__ + b"o^Z",
        )



import Foundation
from PyObjCTools.TestSupport import TestCase, min_sdk_level
import objc


class TestNSXMLParser(TestCase):
    def testConstants(self):
        self.assertEqual(Foundation.NSXMLParserInternalError, 1)
        self.assertEqual(Foundation.NSXMLParserOutOfMemoryError, 2)
        self.assertEqual(Foundation.NSXMLParserDocumentStartError, 3)
        self.assertEqual(Foundation.NSXMLParserEmptyDocumentError, 4)
        self.assertEqual(Foundation.NSXMLParserPrematureDocumentEndError, 5)
        self.assertEqual(Foundation.NSXMLParserInvalidHexCharacterRefError, 6)
        self.assertEqual(Foundation.NSXMLParserInvalidDecimalCharacterRefError, 7)
        self.assertEqual(Foundation.NSXMLParserInvalidCharacterRefError, 8)
        self.assertEqual(Foundation.NSXMLParserInvalidCharacterError, 9)
        self.assertEqual(Foundation.NSXMLParserCharacterRefAtEOFError, 10)
        self.assertEqual(Foundation.NSXMLParserCharacterRefInPrologError, 11)
        self.assertEqual(Foundation.NSXMLParserCharacterRefInEpilogError, 12)
        self.assertEqual(Foundation.NSXMLParserCharacterRefInDTDError, 13)
        self.assertEqual(Foundation.NSXMLParserEntityRefAtEOFError, 14)
        self.assertEqual(Foundation.NSXMLParserEntityRefInPrologError, 15)
        self.assertEqual(Foundation.NSXMLParserEntityRefInEpilogError, 16)
        self.assertEqual(Foundation.NSXMLParserEntityRefInDTDError, 17)
        self.assertEqual(Foundation.NSXMLParserParsedEntityRefAtEOFError, 18)
        self.assertEqual(Foundation.NSXMLParserParsedEntityRefInPrologError, 19)
        self.assertEqual(Foundation.NSXMLParserParsedEntityRefInEpilogError, 20)
        self.assertEqual(Foundation.NSXMLParserParsedEntityRefInInternalSubsetError, 21)
        self.assertEqual(Foundation.NSXMLParserEntityReferenceWithoutNameError, 22)
        self.assertEqual(Foundation.NSXMLParserEntityReferenceMissingSemiError, 23)
        self.assertEqual(Foundation.NSXMLParserParsedEntityRefNoNameError, 24)
        self.assertEqual(Foundation.NSXMLParserParsedEntityRefMissingSemiError, 25)
        self.assertEqual(Foundation.NSXMLParserUndeclaredEntityError, 26)
        self.assertEqual(Foundation.NSXMLParserUnparsedEntityError, 28)
        self.assertEqual(Foundation.NSXMLParserEntityIsExternalError, 29)
        self.assertEqual(Foundation.NSXMLParserEntityIsParameterError, 30)
        self.assertEqual(Foundation.NSXMLParserUnknownEncodingError, 31)
        self.assertEqual(Foundation.NSXMLParserEncodingNotSupportedError, 32)
        self.assertEqual(Foundation.NSXMLParserStringNotStartedError, 33)
        self.assertEqual(Foundation.NSXMLParserStringNotClosedError, 34)
        self.assertEqual(Foundation.NSXMLParserNamespaceDeclarationError, 35)
        self.assertEqual(Foundation.NSXMLParserEntityNotStartedError, 36)
        self.assertEqual(Foundation.NSXMLParserEntityNotFinishedError, 37)
        self.assertEqual(Foundation.NSXMLParserLessThanSymbolInAttributeError, 38)
        self.assertEqual(Foundation.NSXMLParserAttributeNotStartedError, 39)
        self.assertEqual(Foundation.NSXMLParserAttributeNotFinishedError, 40)
        self.assertEqual(Foundation.NSXMLParserAttributeHasNoValueError, 41)
        self.assertEqual(Foundation.NSXMLParserAttributeRedefinedError, 42)
        self.assertEqual(Foundation.NSXMLParserLiteralNotStartedError, 43)
        self.assertEqual(Foundation.NSXMLParserLiteralNotFinishedError, 44)
        self.assertEqual(Foundation.NSXMLParserCommentNotFinishedError, 45)
        self.assertEqual(Foundation.NSXMLParserProcessingInstructionNotStartedError, 46)
        self.assertEqual(
            Foundation.NSXMLParserProcessingInstructionNotFinishedError, 47
        )
        self.assertEqual(Foundation.NSXMLParserNotationNotStartedError, 48)
        self.assertEqual(Foundation.NSXMLParserNotationNotFinishedError, 49)
        self.assertEqual(Foundation.NSXMLParserAttributeListNotStartedError, 50)
        self.assertEqual(Foundation.NSXMLParserAttributeListNotFinishedError, 51)
        self.assertEqual(Foundation.NSXMLParserMixedContentDeclNotStartedError, 52)
        self.assertEqual(Foundation.NSXMLParserMixedContentDeclNotFinishedError, 53)
        self.assertEqual(Foundation.NSXMLParserElementContentDeclNotStartedError, 54)
        self.assertEqual(Foundation.NSXMLParserElementContentDeclNotFinishedError, 55)
        self.assertEqual(Foundation.NSXMLParserXMLDeclNotStartedError, 56)
        self.assertEqual(Foundation.NSXMLParserXMLDeclNotFinishedError, 57)
        self.assertEqual(Foundation.NSXMLParserConditionalSectionNotStartedError, 58)
        self.assertEqual(Foundation.NSXMLParserConditionalSectionNotFinishedError, 59)
        self.assertEqual(Foundation.NSXMLParserExternalSubsetNotFinishedError, 60)
        self.assertEqual(Foundation.NSXMLParserDOCTYPEDeclNotFinishedError, 61)
        self.assertEqual(Foundation.NSXMLParserMisplacedCDATAEndStringError, 62)
        self.assertEqual(Foundation.NSXMLParserCDATANotFinishedError, 63)
        self.assertEqual(Foundation.NSXMLParserMisplacedXMLDeclarationError, 64)
        self.assertEqual(Foundation.NSXMLParserSpaceRequiredError, 65)
        self.assertEqual(Foundation.NSXMLParserSeparatorRequiredError, 66)
        self.assertEqual(Foundation.NSXMLParserNMTOKENRequiredError, 67)
        self.assertEqual(Foundation.NSXMLParserNAMERequiredError, 68)
        self.assertEqual(Foundation.NSXMLParserPCDATARequiredError, 69)
        self.assertEqual(Foundation.NSXMLParserURIRequiredError, 70)
        self.assertEqual(Foundation.NSXMLParserPublicIdentifierRequiredError, 71)
        self.assertEqual(Foundation.NSXMLParserLTRequiredError, 72)
        self.assertEqual(Foundation.NSXMLParserGTRequiredError, 73)
        self.assertEqual(Foundation.NSXMLParserLTSlashRequiredError, 74)
        self.assertEqual(Foundation.NSXMLParserEqualExpectedError, 75)
        self.assertEqual(Foundation.NSXMLParserTagNameMismatchError, 76)
        self.assertEqual(Foundation.NSXMLParserUnfinishedTagError, 77)
        self.assertEqual(Foundation.NSXMLParserStandaloneValueError, 78)
        self.assertEqual(Foundation.NSXMLParserInvalidEncodingNameError, 79)
        self.assertEqual(Foundation.NSXMLParserCommentContainsDoubleHyphenError, 80)
        self.assertEqual(Foundation.NSXMLParserInvalidEncodingError, 81)
        self.assertEqual(Foundation.NSXMLParserExternalStandaloneEntityError, 82)
        self.assertEqual(Foundation.NSXMLParserInvalidConditionalSectionError, 83)
        self.assertEqual(Foundation.NSXMLParserEntityValueRequiredError, 84)
        self.assertEqual(Foundation.NSXMLParserNotWellBalancedError, 85)
        self.assertEqual(Foundation.NSXMLParserExtraContentError, 86)
        self.assertEqual(Foundation.NSXMLParserInvalidCharacterInEntityError, 87)
        self.assertEqual(Foundation.NSXMLParserParsedEntityRefInInternalError, 88)
        self.assertEqual(Foundation.NSXMLParserEntityRefLoopError, 89)
        self.assertEqual(Foundation.NSXMLParserEntityBoundaryError, 90)
        self.assertEqual(Foundation.NSXMLParserInvalidURIError, 91)
        self.assertEqual(Foundation.NSXMLParserURIFragmentError, 92)
        self.assertEqual(Foundation.NSXMLParserNoDTDError, 94)
        self.assertEqual(Foundation.NSXMLParserDelegateAbortedParseError, 512)

        self.assertIsInstance(Foundation.NSXMLParserErrorDomain, str)

        self.assertEqual(Foundation.NSXMLParserResolveExternalEntitiesNever, 0)
        self.assertEqual(Foundation.NSXMLParserResolveExternalEntitiesNoNetwork, 1)
        self.assertEqual(Foundation.NSXMLParserResolveExternalEntitiesSameOriginOnly, 2)
        self.assertEqual(Foundation.NSXMLParserResolveExternalEntitiesAlways, 3)

    def testMethods(self):
        self.assertArgIsBOOL(Foundation.NSXMLParser.setShouldProcessNamespaces_, 0)
        self.assertArgIsBOOL(
            Foundation.NSXMLParser.setShouldReportNamespacePrefixes_, 0
        )
        self.assertArgIsBOOL(
            Foundation.NSXMLParser.setShouldResolveExternalEntities_, 0
        )
        self.assertResultIsBOOL(Foundation.NSXMLParser.shouldProcessNamespaces)
        self.assertResultIsBOOL(Foundation.NSXMLParser.shouldReportNamespacePrefixes)
        self.assertResultIsBOOL(Foundation.NSXMLParser.shouldResolveExternalEntities)
        self.assertResultIsBOOL(Foundation.NSXMLParser.parse)

    @min_sdk_level("10.6")
    def testProtocols(self):
        objc.protocolNamed("NSXMLParserDelegate")



import Foundation
import objc
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSPathUtilities(TestCase):
    def testSearchPaths(self):
        self.assertTrue(
            Foundation.NSSearchPathForDirectoriesInDomains(
                Foundation.NSAllLibrariesDirectory, Foundation.NSAllDomainsMask, objc.NO
            ),
            "NSSearchPathForDirectoriesInDomains() failed to return anything.",
        )

        self.assertArgIsBOOL(Foundation.NSSearchPathForDirectoriesInDomains, 2)

    def testTrue(self):
        for boolVal in (1, 1 == 1, objc.YES, -1):
            self.assertEqual(
                Foundation.NSSearchPathForDirectoriesInDomains(
                    Foundation.NSLibraryDirectory, Foundation.NSUserDomainMask, boolVal
                )[0][0],
                "/",
                boolVal,
            )

    def testFalse(self):
        for boolVal in (0, 1 != 1, objc.NO):
            self.assertNotEqual(
                Foundation.NSSearchPathForDirectoriesInDomains(
                    Foundation.NSLibraryDirectory, Foundation.NSUserDomainMask, boolVal
                )[0][0],
                "/",
                boolVal,
            )

    def testFunctions(self):
        s = Foundation.NSUserName()
        self.assertIsInstance(s, str)
        s = Foundation.NSFullUserName()
        self.assertIsInstance(s, str)
        s = Foundation.NSHomeDirectory()
        self.assertIsInstance(s, str)
        s = Foundation.NSHomeDirectoryForUser("root")
        self.assertIsInstance(s, str)
        s = Foundation.NSTemporaryDirectory()
        self.assertIsInstance(s, str)
        s = Foundation.NSOpenStepRootDirectory()
        self.assertIsInstance(s, str)

    def testConstants(self):
        self.assertEqual(Foundation.NSApplicationDirectory, 1)
        self.assertEqual(Foundation.NSDemoApplicationDirectory, 2)
        self.assertEqual(Foundation.NSDeveloperApplicationDirectory, 3)
        self.assertEqual(Foundation.NSAdminApplicationDirectory, 4)
        self.assertEqual(Foundation.NSLibraryDirectory, 5)
        self.assertEqual(Foundation.NSDeveloperDirectory, 6)
        self.assertEqual(Foundation.NSUserDirectory, 7)
        self.assertEqual(Foundation.NSDocumentationDirectory, 8)
        self.assertEqual(Foundation.NSDocumentDirectory, 9)
        self.assertEqual(Foundation.NSCoreServiceDirectory, 10)
        self.assertEqual(Foundation.NSDesktopDirectory, 12)
        self.assertEqual(Foundation.NSCachesDirectory, 13)
        self.assertEqual(Foundation.NSApplicationSupportDirectory, 14)
        self.assertEqual(Foundation.NSDownloadsDirectory, 15)
        self.assertEqual(Foundation.NSAllApplicationsDirectory, 100)
        self.assertEqual(Foundation.NSAllLibrariesDirectory, 101)

        self.assertEqual(Foundation.NSUserDomainMask, 1)
        self.assertEqual(Foundation.NSLocalDomainMask, 2)
        self.assertEqual(Foundation.NSNetworkDomainMask, 4)
        self.assertEqual(Foundation.NSSystemDomainMask, 8)
        self.assertEqual(Foundation.NSAllDomainsMask, 0x0FFFF)

    @min_os_level("10.6")
    def testConstants10_6(self):
        self.assertEqual(Foundation.NSAutosavedInformationDirectory, 11)

        self.assertEqual(Foundation.NSInputMethodsDirectory, 16)
        self.assertEqual(Foundation.NSMoviesDirectory, 17)
        self.assertEqual(Foundation.NSMusicDirectory, 18)
        self.assertEqual(Foundation.NSPicturesDirectory, 19)
        self.assertEqual(Foundation.NSPrinterDescriptionDirectory, 20)
        self.assertEqual(Foundation.NSSharedPublicDirectory, 21)
        self.assertEqual(Foundation.NSPreferencePanesDirectory, 22)
        self.assertEqual(Foundation.NSItemReplacementDirectory, 99)

    @min_os_level("10.8")
    def testConstants10_8(self):
        self.assertEqual(Foundation.NSApplicationScriptsDirectory, 23)
        self.assertEqual(Foundation.NSTrashDirectory, 102)

    def testMethods(self):
        self.assertResultIsBOOL(Foundation.NSString.isAbsolutePath)
        self.assertArgIsOut(
            Foundation.NSString.completePathIntoString_caseSensitive_matchesIntoArray_filterTypes_,  # noqa: B950
            0,
        )
        self.assertArgIsBOOL(
            Foundation.NSString.completePathIntoString_caseSensitive_matchesIntoArray_filterTypes_,  # noqa: B950
            1,
        )
        self.assertArgIsOut(
            Foundation.NSString.completePathIntoString_caseSensitive_matchesIntoArray_filterTypes_,  # noqa: B950
            2,
        )
        self.assertResultIsBOOL(
            Foundation.NSString.getFileSystemRepresentation_maxLength_
        )
        self.assertArgHasType(
            Foundation.NSString.getFileSystemRepresentation_maxLength_,
            0,
            b"o^" + objc._C_CHAR_AS_TEXT,
        )
        self.assertArgSizeInArg(
            Foundation.NSString.getFileSystemRepresentation_maxLength_, 0, 1
        )



# import classes required to start application
import Converter  # noqa: F401
import CurrencyConvBindingDocument  # noqa: F401
from PyObjCTools import AppHelper

if __name__ == "__main__":
    AppHelper.runEventLoop()



import os

import CoreFoundation
from PyObjCTools.TestSupport import TestCase


class TestPreferences(TestCase):
    def testGetting(self):
        v = CoreFoundation.CFPreferencesCopyAppValue(
            "Default Window Settings", "com.apple.Terminal"
        )
        self.assertIsInstance(v, str)
        self.assertResultIsBOOL(CoreFoundation.CFPreferencesGetAppBooleanValue)
        self.assertArgHasType(CoreFoundation.CFPreferencesGetAppBooleanValue, 2, b"o^Z")
        v, valid = CoreFoundation.CFPreferencesGetAppBooleanValue(
            "SecureKeyboardEntry", "com.apple.Terminal", None
        )
        self.assertTrue(valid)
        self.assertIsInstance(v, bool)
        self.assertArgHasType(CoreFoundation.CFPreferencesGetAppIntegerValue, 2, b"o^Z")
        v, valid = CoreFoundation.CFPreferencesGetAppIntegerValue(
            "ABMetaDataChangeCount", "com.apple.AddressBook", None
        )
        # self.assertTrue(valid)
        self.assertIsInstance(v, int)
        v = CoreFoundation.CFPreferencesCopyValue(
            "Default Window Settings",
            "com.apple.Terminal",
            CoreFoundation.kCFPreferencesCurrentUser,
            CoreFoundation.kCFPreferencesAnyHost,
        )
        self.assertIsInstance(v, str)
        v = CoreFoundation.CFPreferencesCopyMultiple(
            None,
            "com.apple.Terminal",
            CoreFoundation.kCFPreferencesCurrentUser,
            CoreFoundation.kCFPreferencesAnyHost,
        )
        self.assertIsInstance(v, CoreFoundation.CFDictionaryRef)
        v = CoreFoundation.CFPreferencesCopyMultiple(
            ["AutoFocus", "WindowCloseAction"],
            "com.apple.Terminal",
            CoreFoundation.kCFPreferencesCurrentUser,
            CoreFoundation.kCFPreferencesAnyHost,
        )
        self.assertIsInstance(v, CoreFoundation.CFDictionaryRef)
        self.assertResultIsBOOL(CoreFoundation.CFPreferencesAppValueIsForced)
        v = CoreFoundation.CFPreferencesAppValueIsForced(
            "AutoFocus", "com.apple.Terminal"
        )
        self.assertIs(v is True or v, False)

    def testSetting(self):
        prefsFn = os.path.expanduser("~/Library/Preferences/PyObjCTest.plist")
        if os.path.exists(prefsFn):
            os.unlink(prefsFn)

        v = CoreFoundation.CFPreferencesCopyAppValue("PyObjCTestValue", "PyObjCTest")
        self.assertIs(v, None)
        CoreFoundation.CFPreferencesSetAppValue(
            "PyObjCTestValue", "value1", "PyObjCTest"
        )
        v = CoreFoundation.CFPreferencesCopyAppValue("PyObjCTestValue", "PyObjCTest")
        self.assertEqual(v, "value1")
        CoreFoundation.CFPreferencesSetAppValue("PyObjCTestValue", None, "PyObjCTest")
        CoreFoundation.CFPreferencesAddSuitePreferencesToApp(
            "PyObjCTest", CoreFoundation.kCFPreferencesCurrentApplication
        )
        CoreFoundation.CFPreferencesRemoveSuitePreferencesFromApp(
            "PyObjCTest", CoreFoundation.kCFPreferencesCurrentApplication
        )

        ok = CoreFoundation.CFPreferencesAppSynchronize("PyObjCTest")
        self.assertTrue(ok)

        CoreFoundation.CFPreferencesSetValue(
            "PyObjCTestValue2",
            "value2",
            "PyObjCTest",
            CoreFoundation.kCFPreferencesCurrentUser,
            CoreFoundation.kCFPreferencesAnyHost,
        )
        v = CoreFoundation.CFPreferencesCopyValue(
            "PyObjCTestValue2",
            "PyObjCTest",
            CoreFoundation.kCFPreferencesCurrentUser,
            CoreFoundation.kCFPreferencesAnyHost,
        )
        self.assertEqual(v, "value2")
        CoreFoundation.CFPreferencesSetValue(
            "PyObjCTestValue2",
            None,
            "PyObjCTest",
            CoreFoundation.kCFPreferencesCurrentUser,
            CoreFoundation.kCFPreferencesAnyHost,
        )
        v = CoreFoundation.CFPreferencesCopyValue(
            "PyObjCTestValue2",
            "PyObjCTest",
            CoreFoundation.kCFPreferencesCurrentUser,
            CoreFoundation.kCFPreferencesAnyHost,
        )
        self.assertIs(v, None)
        v = CoreFoundation.CFPreferencesCopyValue(
            "PyObjCTestValue2",
            "PyObjCTest",
            CoreFoundation.kCFPreferencesCurrentUser,
            CoreFoundation.kCFPreferencesAnyHost,
        )
        # self.assertIsNot(v, None)
        CoreFoundation.CFPreferencesSetMultiple(
            {"key1": 99, "key2": 42},
            ["PyObjCTestValue"],
            "PyObjCTest",
            CoreFoundation.kCFPreferencesCurrentUser,
            CoreFoundation.kCFPreferencesAnyHost,
        )
        v = CoreFoundation.CFPreferencesCopyValue(
            "PyObjCTestValue",
            "PyObjCTest",
            CoreFoundation.kCFPreferencesCurrentUser,
            CoreFoundation.kCFPreferencesAnyHost,
        )
        self.assertIs(v, None)
        v = CoreFoundation.CFPreferencesCopyValue(
            "key2",
            "PyObjCTest",
            CoreFoundation.kCFPreferencesCurrentUser,
            CoreFoundation.kCFPreferencesAnyHost,
        )
        self.assertEqual(v, 42)
        if os.path.exists(prefsFn):
            os.unlink(prefsFn)
        ok = CoreFoundation.CFPreferencesSynchronize(
            "PyObjCTest",
            CoreFoundation.kCFPreferencesCurrentUser,
            CoreFoundation.kCFPreferencesAnyHost,
        )

        self.assertTrue(ok)

        self.assertResultIsCFRetained(CoreFoundation.CFPreferencesCopyApplicationList)
        apps = CoreFoundation.CFPreferencesCopyApplicationList(
            CoreFoundation.kCFPreferencesCurrentUser,
            CoreFoundation.kCFPreferencesAnyHost,
        )
        self.assertIsInstance(apps, CoreFoundation.CFArrayRef)
        self.assertIn("com.apple.AddressBook", apps)
        self.assertResultIsCFRetained(CoreFoundation.CFPreferencesCopyKeyList)
        keys = CoreFoundation.CFPreferencesCopyKeyList(
            "com.apple.dock",
            CoreFoundation.kCFPreferencesCurrentUser,
            CoreFoundation.kCFPreferencesAnyHost,
        )
        self.assertIsInstance(keys, CoreFoundation.CFArrayRef)
        self.assertTrue("region" in keys or "version" in keys)

    def testConstants(self):
        self.assertIsInstance(CoreFoundation.kCFPreferencesAnyApplication, str)
        self.assertIsInstance(CoreFoundation.kCFPreferencesCurrentApplication, str)
        self.assertIsInstance(CoreFoundation.kCFPreferencesAnyHost, str)
        self.assertIsInstance(CoreFoundation.kCFPreferencesCurrentHost, str)
        self.assertIsInstance(CoreFoundation.kCFPreferencesAnyUser, str)
        self.assertIsInstance(CoreFoundation.kCFPreferencesCurrentUser, str)



import AppKit
import objc
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSTextInputClientHelper(AppKit.NSObject):
    def insertText_replacementRange_(self, txt, rng):
        pass

    def doCommandBySelector_(self, sel):
        pass

    def setMarkedText_selectedRange_replacementRange_(self, txt, rng1, rng2):
        pass

    def selectedRange(self):
        return 1

    def markedRange(self):
        return 1

    def hasMarkedText(self):
        return 1

    def attributedSubstringForProposedRange_actualRange_(self, rng1, rng2):
        return 1

    def firstRectForCharacterRange_actualRange_(self, rng1, rng2):
        return 1

    def characterIndexForPoint_(self, pt):
        return 1

    def fractionOfDistanceThroughGlyphForPoint_(self, pt):
        return 1

    def baselineDeltaForCharacterAtIndex_(self, idx):
        return 1

    def windowLevel(self):
        return 1

    def drawsVerticallyForCharacterAtIndex_(self, i):
        return 1


class TestNSTextInputClient(TestCase):
    @min_os_level("10.5")
    def testMethods(self):
        objc.protocolNamed("NSTextInputClient")
        self.assertArgHasType(
            TestNSTextInputClientHelper.insertText_replacementRange_,
            1,
            AppKit.NSRange.__typestr__,
        )
        self.assertArgHasType(
            TestNSTextInputClientHelper.doCommandBySelector_, 0, objc._C_SEL
        )
        self.assertArgHasType(
            TestNSTextInputClientHelper.setMarkedText_selectedRange_replacementRange_,
            1,
            AppKit.NSRange.__typestr__,
        )
        self.assertArgHasType(
            TestNSTextInputClientHelper.setMarkedText_selectedRange_replacementRange_,
            2,
            AppKit.NSRange.__typestr__,
        )
        self.assertResultHasType(
            TestNSTextInputClientHelper.selectedRange, AppKit.NSRange.__typestr__
        )
        self.assertResultHasType(
            TestNSTextInputClientHelper.markedRange, AppKit.NSRange.__typestr__
        )
        self.assertResultIsBOOL(TestNSTextInputClientHelper.hasMarkedText)
        self.assertArgHasType(
            TestNSTextInputClientHelper.attributedSubstringForProposedRange_actualRange_,
            0,
            AppKit.NSRange.__typestr__,
        )
        self.assertArgHasType(
            TestNSTextInputClientHelper.attributedSubstringForProposedRange_actualRange_,
            1,
            b"o^" + AppKit.NSRange.__typestr__,
        )
        self.assertResultHasType(
            TestNSTextInputClientHelper.firstRectForCharacterRange_actualRange_,
            AppKit.NSRect.__typestr__,
        )
        self.assertArgHasType(
            TestNSTextInputClientHelper.firstRectForCharacterRange_actualRange_,
            0,
            AppKit.NSRange.__typestr__,
        )
        self.assertArgHasType(
            TestNSTextInputClientHelper.firstRectForCharacterRange_actualRange_,
            1,
            b"o^" + AppKit.NSRange.__typestr__,
        )
        self.assertResultHasType(
            TestNSTextInputClientHelper.characterIndexForPoint_, objc._C_NSUInteger
        )
        self.assertArgHasType(
            TestNSTextInputClientHelper.characterIndexForPoint_,
            0,
            AppKit.NSPoint.__typestr__,
        )
        self.assertResultHasType(
            TestNSTextInputClientHelper.fractionOfDistanceThroughGlyphForPoint_,
            objc._C_CGFloat,
        )
        self.assertArgHasType(
            TestNSTextInputClientHelper.fractionOfDistanceThroughGlyphForPoint_,
            0,
            AppKit.NSPoint.__typestr__,
        )
        self.assertResultHasType(
            TestNSTextInputClientHelper.baselineDeltaForCharacterAtIndex_,
            objc._C_CGFloat,
        )
        self.assertArgHasType(
            TestNSTextInputClientHelper.baselineDeltaForCharacterAtIndex_,
            0,
            objc._C_NSUInteger,
        )
        self.assertResultHasType(
            TestNSTextInputClientHelper.windowLevel, objc._C_NSInteger
        )

    @min_os_level("10.6")
    def testMethods10_6(self):
        self.assertResultIsBOOL(
            TestNSTextInputClientHelper.drawsVerticallyForCharacterAtIndex_
        )
        self.assertArgHasType(
            TestNSTextInputClientHelper.drawsVerticallyForCharacterAtIndex_,
            0,
            objc._C_NSInteger,
        )



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSMeasurementFormatter(TestCase):
    @min_os_level("10.12")
    def testConstants(self):
        self.assertEqual(
            Foundation.NSMeasurementFormatterUnitOptionsProvidedUnit, 1 << 0
        )
        self.assertEqual(
            Foundation.NSMeasurementFormatterUnitOptionsNaturalScale, 1 << 1
        )
        self.assertEqual(
            Foundation.NSMeasurementFormatterUnitOptionsTemperatureWithoutUnit, 1 << 2
        )



#
#  Category.py
#  ToDos
#
#  Converted by u.fiedler on 09.02.05.
#
#  The original version was written in Objective-C by Malcolm Crawford
#  at http://homepage.mac.com/mmalc/CocoaExamples/controllers.html

import objc
from Foundation import NSObject


class Category(NSObject):
    title = objc.ivar("title")
    priority = objc.ivar("priority", objc._C_INT)

    @classmethod
    def allCategories(cls):
        """Predefined global list of categories"""
        return categories

    @classmethod
    def categoryForPriority_(cls, thePriority):
        for category in categories:
            if thePriority >= category.priority:
                return category
        return None

    @classmethod
    def categoryWithTitle_andPriority_(cls, aTitle, aValue):
        """Convenience constructor"""
        newCategory = Category.alloc().init()
        newCategory.title = aTitle
        newCategory.priority = aValue
        return newCategory

    # NSCoding methods
    # To encode, simply save 'priority'; on decode, replace self with
    # the existing instance from 'allCategories' with the same priority

    def encodeWithCoder_(self, encoder):
        if encoder.allowsKeyedCoding():
            encoder.encodeInt_forKey_(self.priority, "priority")
        else:
            encoder.encodeObject_(self.priority)

    def initWithCoder_(self, decoder):
        if decoder.allowsKeyedCoding():
            thePriority = decoder.decodeIntForKey_("priority")
        else:
            thePriority = decoder.decodeObject()
        return Category.categoryForPriority_(thePriority)


categories = [
    Category.categoryWithTitle_andPriority_("Vital", 11),
    Category.categoryWithTitle_andPriority_("Very Important", 4),
    Category.categoryWithTitle_andPriority_("Important", 3),
    Category.categoryWithTitle_andPriority_("Not Important", 2),
    Category.categoryWithTitle_andPriority_("Whenever", 0),
]



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level
import objc


class TestNSPanel(TestCase):
    def testFunctions(self):
        self.assertArgIsPrintf(AppKit.NSRunAlertPanel, 1)
        self.assertArgIsPrintf(AppKit.NSRunInformationalAlertPanel, 1)
        self.assertArgIsPrintf(AppKit.NSRunCriticalAlertPanel, 1)
        self.assertArgIsPrintf(AppKit.NSRunAlertPanelRelativeToWindow, 1)
        self.assertArgIsPrintf(AppKit.NSRunInformationalAlertPanelRelativeToWindow, 1)
        self.assertArgIsPrintf(AppKit.NSRunCriticalAlertPanelRelativeToWindow, 1)
        self.assertArgIsPrintf(AppKit.NSBeginAlertSheet, 9)
        self.assertArgIsSEL(
            AppKit.NSBeginAlertSheet, 6, b"v@:@" + objc._C_NSInteger + b"^v"
        )
        self.assertArgIsSEL(
            AppKit.NSBeginAlertSheet, 7, b"v@:@" + objc._C_NSInteger + b"^v"
        )
        self.assertArgHasType(AppKit.NSBeginAlertSheet, 8, b"^v")
        self.assertArgIsPrintf(AppKit.NSBeginInformationalAlertSheet, 9)
        self.assertArgIsSEL(
            AppKit.NSBeginInformationalAlertSheet,
            6,
            b"v@:@" + objc._C_NSInteger + b"^v",
        )
        self.assertArgIsSEL(
            AppKit.NSBeginInformationalAlertSheet,
            7,
            b"v@:@" + objc._C_NSInteger + b"^v",
        )
        self.assertArgHasType(AppKit.NSBeginInformationalAlertSheet, 8, b"^v")
        self.assertArgIsPrintf(AppKit.NSBeginCriticalAlertSheet, 9)
        self.assertArgIsSEL(
            AppKit.NSBeginCriticalAlertSheet, 6, b"v@:@" + objc._C_NSInteger + b"^v"
        )
        self.assertArgIsSEL(
            AppKit.NSBeginCriticalAlertSheet, 7, b"v@:@" + objc._C_NSInteger + b"^v"
        )
        self.assertArgHasType(AppKit.NSBeginCriticalAlertSheet, 8, b"^v")
        self.assertArgIsPrintf(AppKit.NSGetAlertPanel, 1)
        self.assertArgIsPrintf(AppKit.NSGetInformationalAlertPanel, 1)
        self.assertArgIsPrintf(AppKit.NSGetCriticalAlertPanel, 1)

        panel = AppKit.NSGetInformationalAlertPanel(
            "title", "fmt %d", "ok", "cancel", "help", 10
        )
        self.assertIsInstance(panel, AppKit.NSPanel)

        AppKit.NSReleaseAlertPanel(panel)

    def testConstants(self):
        self.assertEqual(AppKit.NSAlertDefaultReturn, 1)
        self.assertEqual(AppKit.NSAlertAlternateReturn, 0)
        self.assertEqual(AppKit.NSAlertOtherReturn, -1)
        self.assertEqual(AppKit.NSAlertErrorReturn, -2)
        self.assertEqual(AppKit.NSOKButton, 1)
        self.assertEqual(AppKit.NSCancelButton, 0)
        self.assertEqual(AppKit.NSUtilityWindowMask, 1 << 4)
        self.assertEqual(AppKit.NSDocModalWindowMask, 1 << 6)
        self.assertEqual(AppKit.NSNonactivatingPanelMask, 1 << 7)

    @min_os_level("10.5")
    def testConstants10_5(self):
        self.assertEqual(AppKit.NSHUDWindowMask, 1 << 13)

    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSPanel.isFloatingPanel)
        self.assertArgIsBOOL(AppKit.NSPanel.setFloatingPanel_, 0)
        self.assertResultIsBOOL(AppKit.NSPanel.becomesKeyOnlyIfNeeded)
        self.assertArgIsBOOL(AppKit.NSPanel.setBecomesKeyOnlyIfNeeded_, 0)
        self.assertResultIsBOOL(AppKit.NSPanel.worksWhenModal)
        self.assertArgIsBOOL(AppKit.NSPanel.setWorksWhenModal_, 0)

    @min_os_level("10.11")
    def testMethods10_11(self):
        self.assertResultIsBOOL(AppKit.NSOpenPanel.isAccessoryViewDisclosed)
        self.assertArgIsBOOL(AppKit.NSOpenPanel.setAccessoryViewDisclosed_, 0)



import Foundation
from PyObjCTools.TestSupport import TestCase


class TestNSAutoreleasePoolInteraction(TestCase):
    def testNSAutoreleasePoolPlain(self):
        pool = Foundation.NSAutoreleasePool.alloc().init()
        bar = Foundation.NSMutableArray.array()
        del pool  # Always use 'del pool' instead of 'pool.release()'!
        bar.addObject_("a")  # should still exist because of python GC



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSProxyHelper(Foundation.NSObject):
    def allowsWeakReference(self):
        return 0

    def retainWeakReference(self):
        return 0


class TestNSProxy(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(Foundation.NSProxy.respondsToSelector_)

    @min_os_level("10.7")
    def testMethods10_7(self):
        self.assertResultIsBOOL(TestNSProxyHelper.allowsWeakReference)
        self.assertResultIsBOOL(TestNSProxyHelper.retainWeakReference)
